import { Cloneable } from '../../shared/cloneable.js';
/**
 * Basic building block of beatmap item.
 *
 * Items contains basic information and utilities.
 *
 * @abstract
 */
export class BaseItem extends Cloneable {
    constructor() {
        super(...arguments);
        Object.defineProperty(this, "customData", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
    }
    /**
     * Convert object into class.
     *
     * **WARNING:** If you ever need to use this,
     * you may have used function that used wrapper interface and add onto array/object when are supposed to be concrete.
     */
    reconcile() {
        return this;
    }
    setCustomData(value) {
        this.customData = value;
        return this;
    }
    resetCustomData() {
        this.customData = {};
        return this;
    }
    removeCustomData(key) {
        if (typeof key === 'string') {
            delete this.customData[key];
        }
        else {
            key.forEach((k) => delete this.customData[k]);
        }
        return this;
    }
    addCustomData(object) {
        for (const k in object) {
            this.customData[k] = object[k];
        }
        return this;
    }
    /**
     * Sort beatmap object(s) accordingly.
     *
     * Certain objects may not contain sortable array.
     *
     * Custom function can be provided and will run after base implemention.
     */
    sort(fn) {
        fn?.(this);
        return this;
    }
    /** Allow for advanced custom function. */
    perform(fn) {
        fn(this);
        return this;
    }
    /**
     * Check if object is valid in vanilla game.
     * This may also allow for value outside of intended value as long as it does not cause
     * weird, inconsistent behaviour or unable to load the map.
     *
     * However, this validity may not be applicable to older version of the game or schema.
     *
     * Override allow for custom function to take over rather than run alongside.
     *
     * @example
     * ```ts
     * if (!obj.isValid(optionalFn)) {}
     * ```
     */
    isValid(fn, _override) {
        return fn ? !!fn(this) : true;
    }
    /**
     * Check an object given a custom function.
     *
     * @example
     * ```ts
     * if (obj.check(hasChromaFn)) {}
     * ```
     */
    check(fn) {
        return fn ? !!fn(this) : false;
    }
}
