import { isBlueNoteColor, isDouble, isRedNoteColor, mirrorNoteColor, mirrorNoteDirectionHorizontally, resolveNoteAngle, } from '../../helpers/core/baseNote.js';
import { GridObject } from './gridObject.js';
/**
 * Base note beatmap object.
 *
 * @abstract
 */
export class BaseNote extends GridObject {
    constructor() {
        super(...arguments);
        Object.defineProperty(this, "color", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "direction", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
    }
    setColor(value) {
        this.color = value;
        return this;
    }
    setDirection(value) {
        this.direction = value;
        return this;
    }
    mirror(flipColor = true, fn) {
        fn?.(this);
        if (flipColor)
            this.color = mirrorNoteColor(this.color);
        this.direction = mirrorNoteDirectionHorizontally(this.direction);
        return super.mirror(flipColor);
    }
    /**
     * Check if note is red note.
     * ```ts
     * if (note.isBlue()) {}
     * ```
     */
    isRed() {
        return isRedNoteColor(this.color);
    }
    /**
     * Check if note is blue note.
     * ```ts
     * if (note.isBlue()) {}
     * ```
     */
    isBlue() {
        return isBlueNoteColor(this.color);
    }
    /**
     * Get standardised note angle.
     *
     * @example
     * ```ts
     * import type { IBaseNote } from './baseNote.ts';
     * let note!: IBaseNote;
     * const optionalFn = (object: IBaseNote) => object.customData.value;
     * const tailAngle = note.getTailAngle(optionalFn);
     * ```
     *
     * Custom function are used to return any arbitrary data first if value exist, otherwise returns base value.
     */
    getAngle(fn) {
        return fn?.(this) ?? resolveNoteAngle(this.direction);
    }
    /**
     * Compare current note with the note ahead of it and return if the notes is a double.
     * ```ts
     * if (note.isDouble(otherNote, tol, optionalFn)) {}
     * ```
     */
    isDouble(compareTo, tolerance = 0.01) {
        return isDouble(this, compareTo, tolerance);
    }
    isValid(fn, override) {
        return override
            ? super.isValid(fn, override)
            : super.isValid(fn, override) && this.direction >= 0 && this.direction <= 8;
    }
}
