import { deepCopy } from '../../utils/misc/json.js';
import { isBlueEventValue, isBpmChangeEventType, isColorBoostEventType, isExtraEventType, isFadeEventValue, isFlashEventValue, isLaneRotationEventType, isLaserRotationEventType, isLightEventType, isLightingEventType, isNjsChangeEventType, isOffEventValue, isOldChromaEventValue, isOnEventValue, isRedEventValue, isRingEventType, isSpecialEventType, isTransitionEventValue, isValidEventType, isWhiteEventValue, } from '../helpers/core/basicEvent.js';
import { BaseObject } from './abstract/baseObject.js';
export function createBasicEvent(data = {}) {
    return {
        time: data.time ?? 0,
        type: data.type ?? 0,
        value: data.value ?? 0,
        floatValue: data.floatValue ?? 0,
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap basic event.
 */
export class BasicEvent extends BaseObject {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "type", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "value", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "floatValue", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.time = data.time ?? BasicEvent.defaultValue.time;
        this.type = data.type ?? BasicEvent.defaultValue.type;
        this.value = data.value ?? BasicEvent.defaultValue.value;
        this.floatValue = data.floatValue ?? BasicEvent.defaultValue.floatValue;
        this.customData = deepCopy(data.customData ?? BasicEvent.defaultValue.customData);
    }
    /**
     * Check if event is a valid type.
     * ```ts
     * if (event.isValidType()) {}
     * ```
     */
    isValidType() {
        return isValidEventType(this.type);
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            this.isValidType() &&
            this.value >= 0 &&
            !(!this.isLaserRotationEvent() && this.value > 12 && !this.isOldChroma());
    }
    setType(value) {
        this.type = value;
        return this;
    }
    setValue(value) {
        this.value = value;
        return this;
    }
    setFloatValue(value) {
        this.floatValue = value;
        return this;
    }
    /**
     * Check if light event is an off event.
     * ```ts
     * if (event.isOff()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isOff() {
        return isOffEventValue(this.value);
    }
    /**
     * Check if light event is an on event.
     * ```ts
     * if (event.isOn()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isOn() {
        return isOnEventValue(this.value);
    }
    /**
     * Check if light event is a flash event.
     * ```ts
     * if (event.isFlash()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isFlash() {
        return isFlashEventValue(this.value);
    }
    /**
     * Check if light event is a fade event.
     * ```ts
     * if (event.isFade()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isFade() {
        return isFadeEventValue(this.value);
    }
    /**
     * Check if light event is a transition event.
     * ```ts
     * if (event.isTransition()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isTransition() {
        return isTransitionEventValue(this.value);
    }
    /**
     * Check if light event is a blue light.
     * ```ts
     * if (event.isBlue()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isBlue() {
        return isBlueEventValue(this.value);
    }
    /**
     * Check if light event is a red light.
     * ```ts
     * if (event.isRed()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isRed() {
        return isRedEventValue(this.value);
    }
    /**
     * Check if light event is a white light.
     * ```ts
     * if (event.isWhite()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isWhite() {
        return isWhiteEventValue(this.value);
    }
    /**
     * Check if event is a light event.
     * ```ts
     * if (event.isLightEvent()) {}
     * ```
     */
    isLightEvent(environment) {
        return isLightEventType(this.type, environment);
    }
    /**
     * Check if event is a boost event.
     * ```ts
     * if (event.isColorBoost()) {}
     * ```
     */
    isColorBoost() {
        return isColorBoostEventType(this.type);
    }
    /**
     * Check if event is a ring event.
     * ```ts
     * if (event.isRingEvent()) {}
     * ```
     *
     * This does not check for ring zoom.
     */
    isRingEvent(environment) {
        return isRingEventType(this.type, environment);
    }
    /**
     * Check if event is a laser rotation event.
     * ```ts
     * if (event.isLaserRotationEvent()) {}
     * ```
     */
    isLaserRotationEvent(environment) {
        return isLaserRotationEventType(this.type, environment);
    }
    /**
     * Check if event is a lane rotation event.
     * ```ts
     * if (event.isLaneRotationEvent()) {}
     * ```
     */
    isLaneRotationEvent() {
        return isLaneRotationEventType(this.type);
    }
    /**
     * Check if event is a extra event.
     * ```ts
     * if (event.isExtraEvent()) {}
     * ```
     */
    isExtraEvent(environment) {
        return isExtraEventType(this.type, environment);
    }
    /**
     * Check if event is a special event.
     * ```ts
     * if (event.isSpecialEvent()) {}
     * ```
     */
    isSpecialEvent(environment) {
        return isSpecialEventType(this.type, environment);
    }
    /**
     * Check if event is a BPM change event.
     * ```ts
     * if (event.isBpmEvent()) {}
     * ```
     */
    isBpmEvent() {
        return isBpmChangeEventType(this.type);
    }
    isNjsEvent() {
        return isNjsChangeEventType(this.type);
    }
    /**
     * Not to be confused with `isLightEvent`, this checks for event that affects the environment/lighting.
     * ```ts
     * if (event.isLightingEvent()) {}
     * ```
     */
    isLightingEvent() {
        return isLightingEventType(this.type);
    }
    /**
     * Check if event has old Chroma properties.
     * ```ts
     * if (event.isOldChroma()) {}
     * ```
     */
    isOldChroma() {
        return isOldChromaEventValue(this.value);
    }
}
Object.defineProperty(BasicEvent, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createBasicEvent()
});
