import { deepCopy } from '../../utils/misc/json.js';
import { BaseItem } from './abstract/baseItem.js';
import { createDifficulty, Difficulty } from './difficulty.js';
import { createLightshow, Lightshow } from './lightshow.js';
export function createBeatmap(data = {}) {
    return {
        version: data.version ?? -1,
        filename: data.filename ?? 'Unnamed.beatmap.dat',
        lightshowFilename: data.lightshowFilename ?? 'Unnamed.lightshow.dat',
        difficulty: createDifficulty(data.difficulty),
        lightshow: createLightshow(data.lightshow),
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap container.
 *
 * Contains `Difficulty` and `Lightshow`.
 * This object is writable into 2 files.
 * Both may be combined given schema.
 *
 * **WARN:** Do not use `customData` for anything related to either `difficulty` or `lightshow`,
 * access them directly from `difficulty` and `lightshow` if you wish to modify them.
 * This object exists solely for arbitrary data.
 */
export class Beatmap extends BaseItem {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "version", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "difficulty", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lightshow", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "filename", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lightshowFilename", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.version = data.version ?? Beatmap.defaultValue.version;
        this.filename = data.filename ?? Beatmap.defaultValue.filename;
        this.lightshowFilename = data.lightshowFilename ?? Beatmap.defaultValue.lightshowFilename;
        this.difficulty = new Difficulty(data.difficulty ?? Beatmap.defaultValue.difficulty);
        this.lightshow = new Lightshow(data.lightshow ?? Beatmap.defaultValue.lightshow);
    }
    isValid(fn, _override) {
        return super.isValid(fn);
    }
    get bpmEvents() {
        return this.difficulty.bpmEvents;
    }
    set bpmEvents(value) {
        this.difficulty.bpmEvents = value;
    }
    get rotationEvents() {
        return this.difficulty.rotationEvents;
    }
    set rotationEvents(value) {
        this.difficulty.rotationEvents = value;
    }
    get colorNotes() {
        return this.difficulty.colorNotes;
    }
    set colorNotes(value) {
        this.difficulty.colorNotes = value;
    }
    get bombNotes() {
        return this.difficulty.bombNotes;
    }
    set bombNotes(value) {
        this.difficulty.bombNotes = value;
    }
    get obstacles() {
        return this.difficulty.obstacles;
    }
    set obstacles(value) {
        this.difficulty.obstacles = value;
    }
    get arcs() {
        return this.difficulty.arcs;
    }
    set arcs(value) {
        this.difficulty.arcs = value;
    }
    get chains() {
        return this.difficulty.chains;
    }
    set chains(value) {
        this.difficulty.chains = value;
    }
    get njsEvents() {
        return this.difficulty.njsEvents;
    }
    set njsEvents(value) {
        this.difficulty.njsEvents = value;
    }
    get waypoints() {
        return this.lightshow.waypoints;
    }
    set waypoints(value) {
        this.lightshow.waypoints = value;
    }
    get basicEvents() {
        return this.lightshow.basicEvents;
    }
    set basicEvents(value) {
        this.lightshow.basicEvents = value;
    }
    get colorBoostEvents() {
        return this.lightshow.colorBoostEvents;
    }
    set colorBoostEvents(value) {
        this.lightshow.colorBoostEvents = value;
    }
    get lightColorEventBoxGroups() {
        return this.lightshow.lightColorEventBoxGroups;
    }
    set lightColorEventBoxGroups(value) {
        this.lightshow.lightColorEventBoxGroups = value;
    }
    get lightRotationEventBoxGroups() {
        return this.lightshow.lightRotationEventBoxGroups;
    }
    set lightRotationEventBoxGroups(value) {
        this.lightshow.lightRotationEventBoxGroups = value;
    }
    get lightTranslationEventBoxGroups() {
        return this.lightshow.lightTranslationEventBoxGroups;
    }
    set lightTranslationEventBoxGroups(value) {
        this.lightshow.lightTranslationEventBoxGroups = value;
    }
    get fxEventBoxGroups() {
        return this.lightshow.fxEventBoxGroups;
    }
    set fxEventBoxGroups(value) {
        this.lightshow.fxEventBoxGroups = value;
    }
    get basicEventTypesWithKeywords() {
        return this.lightshow.basicEventTypesWithKeywords;
    }
    set basicEventTypesWithKeywords(value) {
        this.lightshow.basicEventTypesWithKeywords = value;
    }
    get useNormalEventsAsCompatibleEvents() {
        return this.lightshow.useNormalEventsAsCompatibleEvents;
    }
    set useNormalEventsAsCompatibleEvents(value) {
        this.lightshow.useNormalEventsAsCompatibleEvents = value;
    }
    setFilename(filename) {
        this.filename = filename;
        return this;
    }
    setLightshowFilename(filename) {
        this.lightshowFilename = filename;
        return this;
    }
    setVersion(version) {
        this.version = version;
        return this;
    }
    sort(fn) {
        this.difficulty.sort();
        this.lightshow.sort();
        return super.sort(fn);
    }
    allObjects() {
        return [
            ...this.difficulty.allObjects(),
            ...this.lightshow.allObjects(),
        ];
    }
    addBpmEvents(...data) {
        this.difficulty.addBpmEvents(...data);
        return this;
    }
    addRotationEvents(...data) {
        this.difficulty.addRotationEvents(...data);
        return this;
    }
    addColorNotes(...data) {
        this.difficulty.addColorNotes(...data);
        return this;
    }
    addBombNotes(...data) {
        this.difficulty.addBombNotes(...data);
        return this;
    }
    addObstacles(...data) {
        this.difficulty.addObstacles(...data);
        return this;
    }
    addArcs(...data) {
        this.difficulty.addArcs(...data);
        return this;
    }
    addChains(...data) {
        this.difficulty.addChains(...data);
        return this;
    }
    addNjsEvents(...data) {
        this.difficulty.addNjsEvents(...data);
        return this;
    }
    addWaypoints(...data) {
        this.lightshow.addWaypoints(...data);
        return this;
    }
    addBasicEvents(...data) {
        this.lightshow.addBasicEvents(...data);
        return this;
    }
    addColorBoostEvents(...data) {
        this.lightshow.addColorBoostEvents(...data);
        return this;
    }
    addLightColorEventBoxGroups(...data) {
        this.lightshow.addLightColorEventBoxGroups(...data);
        return this;
    }
    addLightRotationEventBoxGroups(...data) {
        this.lightshow.addLightRotationEventBoxGroups(...data);
        return this;
    }
    addLightTranslationEventBoxGroups(...data) {
        this.lightshow.addLightTranslationEventBoxGroups(...data);
        return this;
    }
    addFxEventBoxGroups(...data) {
        this.lightshow.addFxEventBoxGroups(...data);
        return this;
    }
}
Object.defineProperty(Beatmap, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createBeatmap()
});
