import { deepCopy } from '../../utils/misc/json.js';
import { reconcileClassObject } from '../helpers/core/misc.js';
import { sortNoteFn, sortObjectFn } from '../helpers/sort.js';
import { BaseItem } from './abstract/baseItem.js';
import { Arc, createArc } from './arc.js';
import { BombNote, createBombNote } from './bombNote.js';
import { BPMEvent, createBPMEvent } from './bpmEvent.js';
import { Chain, createChain } from './chain.js';
import { ColorNote, createColorNote } from './colorNote.js';
import { createNJSEvent, NJSEvent } from './njsEvent.js';
import { createObstacle, Obstacle } from './obstacle.js';
import { createRotationEvent, RotationEvent } from './rotationEvent.js';
export function createDifficulty(data = {}) {
    return {
        bpmEvents: data.bpmEvents?.map(createBPMEvent) ?? [],
        rotationEvents: data.rotationEvents?.map(createRotationEvent) ?? [],
        colorNotes: data.colorNotes?.map(createColorNote) ?? [],
        bombNotes: data.bombNotes?.map(createBombNote) ?? [],
        obstacles: data.obstacles?.map(createObstacle) ?? [],
        arcs: data.arcs?.map(createArc) ?? [],
        chains: data.chains?.map(createChain) ?? [],
        njsEvents: data.njsEvents?.map(createNJSEvent) ?? [],
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap difficulty.
 */
export class Difficulty extends BaseItem {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "bpmEvents", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "rotationEvents", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "colorNotes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "bombNotes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "obstacles", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "arcs", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "chains", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "njsEvents", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "customData", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.bpmEvents = (data.bpmEvents ?? Difficulty.defaultValue.bpmEvents).map((e) => new BPMEvent(e));
        this.rotationEvents = (data.rotationEvents ?? Difficulty.defaultValue.rotationEvents).map((e) => new RotationEvent(e));
        this.colorNotes = (data.colorNotes ?? Difficulty.defaultValue.colorNotes).map((e) => new ColorNote(e));
        this.bombNotes = (data.bombNotes ?? Difficulty.defaultValue.bombNotes).map((e) => new BombNote(e));
        this.obstacles = (data.obstacles ?? Difficulty.defaultValue.obstacles).map((e) => new Obstacle(e));
        this.arcs = (data.arcs ?? Difficulty.defaultValue.arcs).map((e) => new Arc(e));
        this.chains = (data.chains ?? Difficulty.defaultValue.chains).map((e) => new Chain(e));
        this.njsEvents = (data.njsEvents ?? Difficulty.defaultValue.njsEvents).map((e) => new NJSEvent(e));
        this.customData = deepCopy(data.customData ?? Difficulty.defaultValue.customData);
    }
    reconcile() {
        this.bpmEvents = reconcileClassObject(this.bpmEvents, BPMEvent);
        this.rotationEvents = reconcileClassObject(this.rotationEvents, RotationEvent);
        this.colorNotes = reconcileClassObject(this.colorNotes, ColorNote);
        this.bombNotes = reconcileClassObject(this.bombNotes, BombNote);
        this.obstacles = reconcileClassObject(this.obstacles, Obstacle);
        this.arcs = reconcileClassObject(this.arcs, Arc);
        this.chains = reconcileClassObject(this.chains, Chain);
        this.njsEvents = reconcileClassObject(this.njsEvents, NJSEvent);
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            this.bpmEvents.every((e) => e.isValid()) &&
            this.rotationEvents.every((e) => e.isValid()) &&
            this.colorNotes.every((e) => e.isValid()) &&
            this.bombNotes.every((e) => e.isValid()) &&
            this.obstacles.every((e) => e.isValid()) &&
            this.arcs.every((e) => e.isValid()) &&
            this.chains.every((e) => e.isValid());
    }
    sort() {
        this.bpmEvents.sort(sortObjectFn);
        this.rotationEvents.sort(sortObjectFn);
        this.colorNotes.sort(sortNoteFn);
        this.bombNotes.sort(sortNoteFn);
        this.obstacles.sort(sortObjectFn);
        this.arcs.sort(sortNoteFn);
        this.chains.sort(sortNoteFn);
        return this;
    }
    allObjects() {
        return [
            ...this.bpmEvents,
            ...this.rotationEvents,
            ...this.colorNotes,
            ...this.bombNotes,
            ...this.obstacles,
            ...this.arcs,
            ...this.chains,
            ...this.njsEvents,
        ];
    }
    addBpmEvents(...data) {
        for (const d of data) {
            this.bpmEvents.push(new BPMEvent(d));
        }
        return this;
    }
    addRotationEvents(...data) {
        for (const d of data) {
            this.rotationEvents.push(new RotationEvent(d));
        }
        return this;
    }
    addColorNotes(...data) {
        for (const d of data) {
            this.colorNotes.push(new ColorNote(d));
        }
        return this;
    }
    addBombNotes(...data) {
        for (const d of data) {
            this.bombNotes.push(new BombNote(d));
        }
        return this;
    }
    addObstacles(...data) {
        for (const d of data) {
            this.obstacles.push(new Obstacle(d));
        }
        return this;
    }
    addArcs(...data) {
        for (const d of data) {
            this.arcs.push(new Arc(d));
        }
        return this;
    }
    addChains(...data) {
        for (const d of data) {
            this.chains.push(new Chain(d));
        }
        return this;
    }
    addNjsEvents(...data) {
        for (const d of data) {
            this.njsEvents.push(new NJSEvent(d));
        }
        return this;
    }
}
Object.defineProperty(Difficulty, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createDifficulty()
});
