import { deepCopy } from '../../utils/misc/json.js';
import { vectorAdd } from '../../utils/math/vector.js';
import { mirrorCoordinate, resolveGridPosition } from '../helpers/core/gridObject.js';
import { isInteractiveObstacle, isNegativeValueObstacle, isZeroValueObstacle, } from '../helpers/core/obstacle.js';
import { LINE_COUNT } from '../shared/constants.js';
import { GridObject } from './abstract/gridObject.js';
export function createObstacle(data = {}) {
    return {
        time: data.time ?? 0,
        posX: data.posX ?? 0,
        posY: data.posY ?? 0,
        width: data.width ?? 0,
        height: data.height ?? 0,
        duration: data.duration ?? 0,
        laneRotation: data.laneRotation ?? 0,
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap obstacle.
 */
export class Obstacle extends GridObject {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "duration", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "width", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "height", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        this.time = data.time ?? Obstacle.defaultValue.time;
        this.posX = data.posX ?? Obstacle.defaultValue.posX;
        this.posY = data.posY ?? Obstacle.defaultValue.posY;
        this.width = data.width ?? Obstacle.defaultValue.width;
        this.height = data.height ?? Obstacle.defaultValue.height;
        this.duration = data.duration ?? Obstacle.defaultValue.duration;
        this.laneRotation = data.laneRotation ?? Obstacle.defaultValue.laneRotation;
        this.customData = deepCopy(data.customData ?? Obstacle.defaultValue.customData);
    }
    isValid(fn, override) {
        return override
            ? super.isValid(fn, override)
            : super.isValid(fn, override) && !this.hasZero() && !this.hasNegative();
    }
    setDuration(value) {
        this.duration = value;
        return this;
    }
    setWidth(value) {
        this.width = value;
        return this;
    }
    setHeight(value) {
        this.height = value;
        return this;
    }
    mirror(_flipAlt, fn) {
        fn?.(this);
        this.posX = mirrorCoordinate(this.posX + this.width - 1, LINE_COUNT);
        return this;
    }
    getPosition(fn) {
        return fn?.(this) ?? vectorAdd(resolveGridPosition(this), [-2, -0.5]);
    }
    /**
     * Check if obstacle is interactive.
     * ```ts
     * if (wall.isInteractive()) {}
     * ```
     */
    isInteractive() {
        return isInteractiveObstacle(this);
    }
    /**
     * Check if obstacle has zero value.
     * ```ts
     * if (wall.hasZero()) {}
     * ```
     */
    hasZero() {
        return isZeroValueObstacle(this);
    }
    /**
     * Check if obstacle has negative value.
     * ```ts
     * if (wall.hasNegative()) {}
     * ```
     */
    hasNegative() {
        return isNegativeValueObstacle(this);
    }
}
Object.defineProperty(Obstacle, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createObstacle()
});
