/** NJS class for various utility around jump distance, reaction time, etc. */
export class NoteJumpSpeed {
    constructor(bpm, njs = 10, sdm = 0) {
        Object.defineProperty(this, "bpm", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "value", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "offset", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.bpm = bpm;
        this.value = njs;
        this.offset = sdm;
    }
    /**
     * Create and return new instance of NJS class.
     * ```ts
     * const njs = NoteJumpSpeed.create(128, 16, 0);
     * ```
     */
    static create(bpm, njs, sdm) {
        return new NoteJumpSpeed(bpm, njs, sdm);
    }
    get hjd() {
        return this.calcHjd();
    }
    get jd() {
        return this.calcJd();
    }
    get reactionTime() {
        return this.calcRtFromHjd();
    }
    /**
     * Calculate half jump duration given NJS offset.
     * ```ts
     * const hjd = njs.calcHjd();
     * const hjdOffset = njs.calcHjd(0.5);
     * const hjdNoOffset = njs.calcHjd(0);
     * ```
     */
    calcHjd(offset = this.offset) {
        const maxHalfJump = 17.999; // Beat Games, this is not how you fix float inconsistencies
        const num = 60 / this.bpm;
        let hjd = NoteJumpSpeed.HJD_START;
        while (this.value * num * hjd > maxHalfJump)
            hjd /= 2;
        if (hjd < 1)
            hjd = 1;
        return Math.max(hjd + offset, NoteJumpSpeed.HJD_MIN);
    }
    /**
     * Calculate half jump duration given jump distance.
     * ```ts
     * const hjd = njs.calcHjdFromJd();
     * const hjdSpecified = njs.calcHjdFromJd(21);
     * ```
     */
    calcHjdFromJd(jd = this.calcJd()) {
        return jd / ((60 / this.bpm) * this.value * 2);
    }
    /**
     * Calculate half jump duration given real time in second.
     * ```ts
     * const hjd = njs.calcHjdFromRt();
     * const hjdSpecified = njs.calcHjdFromRt(4.5);
     * ```
     */
    calcHjdFromRt(rt = this.calcRtFromHjd()) {
        return rt / (60 / this.bpm);
    }
    /**
     * Calculate offset given half jump duration.
     * ```ts
     * const hjd = njs.calcOffsetFromHjd();
     * ```
     */
    calcOffsetFromHjd(hjd) {
        return hjd - this.calcHjd(0);
    }
    /**
     * Calculate offset given real time in second.
     * ```ts
     * const offset = njs.calcOffsetFromRt();
     * const offsetSpecified = njs.calcOffsetFromRt(4.5);
     * ```
     */
    calcOffsetFromRt(rt = this.calcRtFromHjd()) {
        return rt / (60 / this.bpm) - this.calcHjd(0);
    }
    /**
     * Calculate jump distance given half jump duration.
     * ```ts
     * const jd = njs.calcJd();
     * const jdSpecified = njs.calcJd(1.5);
     * ```
     */
    calcJd(hjd = this.calcHjd()) {
        return this.value * (60 / this.bpm) * hjd * 2;
    }
    /**
     * Calculate optimal jump distance.
     * ```ts
     * const [lowJd, highJd] = njs.calcJdOptimal();
     * ```
     */
    calcJdOptimal() {
        return [
            -(18 / (this.value + 1)) + 18,
            18 * (1 / 1.07) ** this.value + 18,
        ];
    }
    /**
     * Calculate reaction time given jump distance.
     * ```ts
     * const rt = njs.calcRtFromJd();
     * const rtSpecified = njs.calcRtFromJd(21);
     * ```
     */
    calcRtFromJd(jd = this.calcJd()) {
        return jd / (2 * this.value);
    }
    /**
     * Calculate reaction time given half jump duration.
     * ```ts
     * const rt = njs.calcRtFromHjd();
     * const rtSpecified = njs.calcRtFromHjd(1.5);
     * ```
     */
    calcRtFromHjd(hjd = this.calcHjd()) {
        return (60 / this.bpm) * hjd;
    }
    /**
     * Calculate unity distance given beat.
     * ```ts
     * const distance = njs.calcDistance(1);
     * ```
     */
    calcDistance(beat) {
        return ((this.value * 60) / this.bpm) * beat * 2;
    }
}
Object.defineProperty(NoteJumpSpeed, "HJD_START", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: 4
});
Object.defineProperty(NoteJumpSpeed, "HJD_MIN", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: 0.25
});
/**
 * Fallback value used if NJS value is null or 0.
 * ```ts
 * 'Expert+' -> 16,
 * 'ExpertPlus' -> 16,
 * 'Expert' -> 12,
 * 'Hard' -> 10,
 * 'Normal' -> 10,
 * 'Easy' -> 10
 * ```
 */
Object.defineProperty(NoteJumpSpeed, "FallbackNJS", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: {
        'Expert+': 16,
        ExpertPlus: 16,
        Expert: 12,
        Hard: 10,
        Normal: 10,
        Easy: 10,
    }
});
