import { logger } from '../../logger.js';
function tag(name) {
    return ['helpers', 'timeProcessor', name];
}
/** Time processor for various utility around adjusted beat time, JSON time, reaction time, etc. */
export class TimeProcessor {
    constructor(bpm, bpmChange = [], offset = 0) {
        Object.defineProperty(this, "bpm", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_bpmChange", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_timeScale", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_offset", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.bpm = bpm;
        this._offset = offset / 1000;
        this._timeScale = this.getTimeScale(bpmChange.filter((bc) => ('m' in bc || 'bpm' in bc || 'timescale' in bc) && !('o' in bc)));
        this._bpmChange = this.getBpmChangeTime(bpmChange.filter((bc) => ('BPM' in bc && 'metronomeOffset' in bc) ||
            (('_bpm' in bc || '_BPM' in bc) && '_metronomeOffset' in bc) ||
            ('m' in bc && 'o' in bc)));
        if (this._timeScale.length && this._bpmChange.length) {
            logger.tWarn(tag('constructor'), 'BPM change and BPM event should not be used along side together to avoid confusion between editors and in-game behaviour');
        }
    }
    get change() {
        return this._bpmChange;
    }
    set change(val) {
        this._bpmChange = this.getBpmChangeTime(val);
    }
    get timescale() {
        return this._timeScale;
    }
    set timescale(val) {
        this._timeScale = this.getTimeScale(val);
    }
    get offset() {
        return this._offset * 1000;
    }
    set offset(val) {
        this._offset = val / 1000;
    }
    /**
     * Create new BPM change object that allow time to be read according to editor.
     * ```ts
     * const newBpmChange = bpm.getBpmChangeTime(bpmc);
     * ```
     */
    getBpmChangeTime(bpmc = []) {
        let temp;
        const bpmChange = [];
        bpmc = [...bpmc].sort((a, b) => (a.time ??
            a._time ??
            a.b) -
            (b.time ??
                b._time ??
                b.b));
        for (let i = 0; i < bpmc.length; i++) {
            const curBPMC = {
                time: bpmc[i].time ??
                    bpmc[i]._time ??
                    bpmc[i].b,
                BPM: bpmc[i].BPM ??
                    bpmc[i]._BPM ??
                    bpmc[i]._bpm ??
                    bpmc[i].m,
                beatsPerBar: bpmc[i].beatsPerBar ??
                    bpmc[i]._beatsPerBar ??
                    bpmc[i].p,
                metronomeOffset: bpmc[i].metronomeOffset ??
                    bpmc[i]._metronomeOffset ??
                    bpmc[i].o,
                newTime: 0,
            };
            if (temp) {
                curBPMC.newTime = Math.ceil(((curBPMC.time - temp.time) / this.bpm) * temp.BPM +
                    temp.newTime -
                    0.01);
            }
            else {
                curBPMC.newTime = Math.ceil(curBPMC.time - (this._offset * this.bpm) / 60 - 0.01);
            }
            bpmChange.push(curBPMC);
            temp = curBPMC;
        }
        return bpmChange;
    }
    /**
     * Create new time scale object that adjust beat to BPM event.
     * ```ts
     * const newTimeScale = bpm.getTimeScale(bpmc);
     * ```
     */
    getTimeScale(bpmc = []) {
        return [...bpmc]
            .sort((a, b) => (a.time ?? a.b) -
            (b.time ?? b.b))
            .map((el) => {
            if ('scale' in el)
                return el;
            if ('bpm' in el) {
                return { time: el.time, bpm: el.bpm, scale: this.bpm / el.bpm };
            }
            if ('m' in el) {
                return {
                    time: el.b || 0,
                    bpm: el.m || this.bpm,
                    scale: this.bpm / (el.m || this.bpm),
                };
            }
            return { time: el.b || 0, bpm: this.bpm, scale: 1 };
        });
    }
    /**
     * Adjust beat time by offset.
     * ```ts
     * const removedOffset = bpm.offsetBegone(beat);
     * ```
     */
    offsetBegone(beat) {
        if (this._offset === 0)
            return beat;
        return this.toBeatTime(this.toRealTime(beat, false) - this._offset);
    }
    getBpmAtTime(beat) {
        return (this._timeScale.reverse().find((ts) => ts.time <= beat)?.bpm ??
            this.bpm);
    }
    /**
     * Convert beat time at an offset beat to real-time in second.
     * ```ts
     * const durationSecond = bpm.toRealTimeOffset(obstacleTime, obstacleDuration);
     * ```
     */
    toRealTimeAtOffset(offset, beat, timescale = true) {
        if (!timescale) {
            return (beat / this.bpm) * 60;
        }
        for (let i = this._timeScale.length - 1; i >= 0; i--) {
            if (offset > this._timeScale[i].time) {
                return ((beat * this._timeScale[i].scale) / this.bpm) * 60;
            }
        }
        return (beat / this.bpm) * 60;
    }
    /**
     * Convert beat time to real-time in second.
     * ```ts
     * const realTime = bpm.toRealTime(beat);
     * ```
     */
    // this is stupid
    toRealTime(beat, timescale = true) {
        if (!timescale) {
            return (beat / this.bpm) * 60;
        }
        let calculatedBeat = 0;
        for (let i = this._timeScale.length - 1; i >= 0; i--) {
            if (beat > this._timeScale[i].time) {
                calculatedBeat += (beat - this._timeScale[i].time) * this._timeScale[i].scale;
                beat = this._timeScale[i].time;
            }
        }
        return ((beat + calculatedBeat) / this.bpm) * 60;
    }
    /**
     * Convert real-time in second to beat time.
     * ```ts
     * const beatTime = bpm.toBeatTime(rt);
     * ```
     */
    // this is stupid 2 electric boogaloo
    toBeatTime(seconds, timescale = true) {
        if (!timescale) {
            return (seconds * this.bpm) / 60;
        }
        let calculatedSecond = 0;
        for (let i = this._timeScale.length - 1; i >= 0; i--) {
            const currentSeconds = this.toRealTime(this._timeScale[i].time, timescale);
            if (seconds > currentSeconds) {
                calculatedSecond += (seconds - currentSeconds) / this._timeScale[i].scale;
                seconds = currentSeconds;
            }
        }
        return this.toBeatTime(seconds + calculatedSecond, false);
    }
    /**
     * Convert editor beat time to actual JSON time.
     * ```ts
     * const JsonTime = bpm.toJsonTime(beat);
     * ```
     *
     * This only works with custom data BPM change.
     */
    toJsonTime(beat) {
        for (let i = this._bpmChange.length - 1; i >= 0; i--) {
            if (beat > this._bpmChange[i].newTime) {
                return (((beat - this._bpmChange[i].newTime) / this._bpmChange[i].BPM) *
                    this.bpm +
                    this._bpmChange[i].time);
            }
        }
        return this.toBeatTime(this.toRealTime(beat, false) + this._offset);
    }
    /**
     * Adjust beat time from BPM changes and offset.
     * ```ts
     * const adjustedBeat = bpm.adjustTime(beat);
     * ```
     *
     * @deprecated only use if you are using info custom data song offset, typically exist in older v2 beatmaps
     */
    adjustTime(beat) {
        for (let i = this._bpmChange.length - 1; i >= 0; i--) {
            if (beat > this._bpmChange[i].time) {
                return (((beat - this._bpmChange[i].time) / this.bpm) *
                    this._bpmChange[i].BPM +
                    this._bpmChange[i].newTime);
            }
        }
        return this.offsetBegone(beat);
    }
}
/**
 * Create and return an instance of BeatPerMinute class.
 * ```ts
 * const bpm = BeatPerMinute.create(bpm, bpmc, 0);
 * ```
 */
Object.defineProperty(TimeProcessor, "create", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: (bpm, bpmChange, offset) => {
        return new TimeProcessor(bpm, bpmChange, offset);
    }
});
