// deno-lint-ignore-file no-explicit-any
import { logger } from '../../logger.js';
import { implicitVersion, retrieveVersion } from '../helpers/version.js';
import { convertBeatmap } from '../mapping/converter.js';
import { deserializeBeatmap } from '../mapping/schema.js';
import { validateJSON } from '../validator/json.js';
export function tag(name) {
    return ['loader', name];
}
const defaultOptions = {
    forceConvert: true,
    schemaCheck: {},
    sort: true,
    preprocess: [],
    postprocess: [],
};
export function loadBeatmap(type, json, version, options = {}) {
    const optD = (typeof version !== 'number' ? version : options) ?? options ?? {};
    const opt = {
        forceConvert: optD.forceConvert ?? defaultOptions.forceConvert,
        schemaCheck: { ...defaultOptions.schemaCheck, ...optD.schemaCheck },
        sort: optD.sort ?? defaultOptions.sort,
        preprocess: optD.preprocess ?? defaultOptions.preprocess,
        postprocess: optD.postprocess ?? defaultOptions.postprocess,
    };
    const [pretransformer, ...preprocesses] = opt.preprocess;
    let serial = pretransformer
        ? pretransformer(json, version)
        : json;
    preprocesses.forEach((fn, i) => {
        logger.tInfo(tag('loadBeatmap'), 'Running preprocess function #' + (i + 1));
        serial = fn(serial);
    });
    const jsonVerStr = retrieveVersion(serial)?.at(0);
    let jsonVer;
    if (jsonVerStr) {
        jsonVer = parseInt(jsonVerStr);
    }
    else {
        jsonVer = +implicitVersion(type).at(0);
        logger.tWarn(tag('loadBeatmap'), 'Could not identify beatmap version from JSON, assume implicit version', jsonVer);
    }
    let attribute;
    if (opt.schemaCheck.enabled)
        validateJSON(type, serial, jsonVer, opt.schemaCheck);
    attribute = deserializeBeatmap(type, jsonVer, serial);
    if (version && jsonVer !== version) {
        if (!opt.forceConvert) {
            throw new Error(`Beatmap version unmatched, expected ${version} but received ${jsonVer}`);
        }
        logger.tWarn(tag('loadBeatmap'), 'Beatmap version unmatched, expected', version, 'but received', jsonVer, 'for version; Converting to beatmap version', version);
        attribute = convertBeatmap(type, version, attribute, attribute.version);
    }
    if (opt.sort && 'sort' in attribute && typeof attribute.sort === 'function') {
        attribute.sort();
    }
    const [posttransformer, ...postprocesses] = [...opt.postprocess].reverse();
    postprocesses.forEach((fn, i) => {
        logger.tInfo(tag('loadBeatmap'), 'Running postprocess function #' + (i + 1));
        attribute = fn(attribute);
    });
    const wrapper = posttransformer
        ? posttransformer(attribute, version ?? null)
        : attribute;
    return wrapper;
}
