// deno-lint-ignore-file no-explicit-any
import { logger } from '../../logger.js';
import { convertBeatmap } from '../mapping/converter.js';
import { optimizeBeatmap } from '../mapping/optimizer.js';
import { serializeBeatmap } from '../mapping/schema.js';
import { compatibilityCheck } from '../validator/compatibility.js';
import { validateJSON } from '../validator/json.js';
export function tag(name) {
    return ['saver', name];
}
const defaultOptions = {
    format: 0,
    forceConvert: true,
    optimize: {
        enabled: true,
        deduplicate: true,
        floatTrim: 4,
        purgeZeros: true,
        stringTrim: true,
        throwNullish: true,
    },
    validate: { enabled: true },
    sort: true,
    preprocess: [],
    postprocess: [],
};
export function saveBeatmap(type, data, version, options = {}) {
    const optD = (typeof version !== 'number' ? version : options) ?? options ?? {};
    const opt = {
        format: optD.format ?? defaultOptions.format,
        forceConvert: optD.forceConvert ?? defaultOptions.forceConvert,
        optimize: { ...defaultOptions.optimize, ...optD.optimize },
        validate: { ...defaultOptions.validate, ...optD.validate },
        sort: optD.sort ?? defaultOptions.sort,
        preprocess: optD.preprocess ?? defaultOptions.preprocess,
        postprocess: optD.postprocess ?? defaultOptions.postprocess,
    };
    const [pretransformer, ...preprocesses] = opt.preprocess;
    let attribute = pretransformer ? pretransformer(data, version) : data;
    preprocesses.forEach((fn, i) => {
        logger.tInfo(tag('saveBeatmap'), 'Running preprocess function #' + (i + 1));
        attribute = fn(attribute);
    });
    let ver;
    if (typeof version === 'number') {
        ver = version;
    }
    else {
        ver = data.version;
        if (ver === -1) {
            throw new Error('Version is not set, prevented from saving.');
        }
        logger.tInfo(tag('saveBeatmap'), 'Implicitly saving ' + type + ' as version', ver);
    }
    if (ver && attribute.version !== ver) {
        if (!opt.forceConvert) {
            throw new Error(`Beatmap version unmatched, expected ${ver} but received ${data.version}`);
        }
        logger.tWarn(tag('saveBeatmap'), 'Beatmap version unmatched, expected', ver, 'but received', data.version, 'for version; Converting to beatmap version', ver);
        attribute = convertBeatmap(type, ver, attribute, attribute.version);
    }
    // TODO: validate beatmap properly
    // if (opt.validate.enabled) {
    //    logger.tInfo(tag('saveBeatmap'), 'Validating beatmap');
    //    if (!data.isValid()) {
    //       logger.tWarn(tag('saveBeatmap'), 'Invalid data detected in beatmap');
    //    }
    // }
    if (opt.sort && 'sort' in attribute && typeof attribute.sort === 'function') {
        logger.tInfo(tag('saveBeatmap'), 'Sorting beatmap objects');
        attribute.sort();
    }
    if (opt.validate.enabled) {
        compatibilityCheck(type, attribute, ver, opt.validate?.compatibility);
    }
    logger.tInfo(tag('saveBeatmap'), 'Serializing beatmap ' + type + ' as JSON');
    let serial = serializeBeatmap(type, ver, attribute);
    if (!serial) {
        throw new Error('Failed to serialize beatmap, version ' + ver + ' is not supported.');
    }
    if (opt.optimize.enabled) {
        logger.tInfo(tag('saveBeatmap'), 'Optimizing beatmap JSON');
        optimizeBeatmap(type, ver, serial, opt.optimize);
    }
    if (opt.validate.enabled) {
        validateJSON(type, serial, ver, opt.validate?.schemaCheck);
    }
    const [posttransformer, ...postprocesses] = [...opt.postprocess].reverse();
    postprocesses.forEach((fn, i) => {
        logger.tInfo(tag('saveBeatmap'), 'Running postprocess function #' + (i + 1));
        serial = fn(serial);
    });
    const json = posttransformer ? posttransformer(serial, ver) : serial;
    return json;
}
