import { deepCopy } from '../../../utils/misc/json.js';
import { createBeatmap } from '../../core/beatmap.js';
import { createDifficulty } from '../../core/difficulty.js';
import { createLightshow } from '../../core/lightshow.js';
import { sortV2NoteFn, sortV2ObjectFn } from '../../helpers/sort.js';
import { compareVersion } from '../../helpers/version.js';
import { arc } from './arc.js';
import { basicEvent } from './basicEvent.js';
import { basicEventTypesWithKeywords } from './basicEventTypesWithKeywords.js';
import { bombNote } from './bombNote.js';
import { bpmEvent } from './bpmEvent.js';
import { colorBoostEvent } from './colorBoostEvent.js';
import { colorNote } from './colorNote.js';
import { obstacle } from './obstacle.js';
import { rotationEvent } from './rotationEvent.js';
import { waypoint } from './waypoint.js';
/**
 * Schema serialization for v2 `Difficulty`.
 */
export const difficulty = {
    serialize(data) {
        return {
            _version: '2.6.0',
            _notes: [
                ...data.difficulty.colorNotes.map((x) => {
                    return colorNote.serialize(x);
                }),
                ...data.difficulty.bombNotes.map((x) => {
                    return bombNote.serialize(x);
                }),
            ].sort(sortV2NoteFn),
            _sliders: data.difficulty.arcs.map((x) => {
                return arc.serialize(x);
            }),
            _obstacles: data.difficulty.obstacles.map((x) => {
                return obstacle.serialize(x);
            }),
            _events: [
                ...data.lightshow.basicEvents.map((x) => {
                    return basicEvent.serialize(x);
                }),
                ...data.lightshow.colorBoostEvents.map((x) => {
                    return colorBoostEvent.serialize(x);
                }),
                ...data.difficulty.rotationEvents.map((x) => {
                    return rotationEvent.serialize(x);
                }),
                ...data.difficulty.bpmEvents.map((x) => {
                    return bpmEvent.serialize(x);
                }),
            ].sort(sortV2ObjectFn),
            _waypoints: data.lightshow.waypoints.map((x) => {
                return waypoint.serialize(x);
            }),
            _specialEventsKeywordFilters: basicEventTypesWithKeywords.serialize(data.lightshow.basicEventTypesWithKeywords),
            _customData: deepCopy(data.difficulty.customData),
        };
    },
    deserialize(data, options) {
        const colorNotes = [];
        const bombNotes = [];
        const _notes = data._notes || [];
        for (let i = 0; i < _notes.length; i++) {
            const obj = _notes[i];
            if (obj?._type === 3) {
                bombNotes.push(bombNote.deserialize(obj));
            }
            else {
                colorNotes.push(colorNote.deserialize(obj));
            }
        }
        const preV25 = compareVersion(data._version || '2.0.0', '2.5.0');
        const basicEvents = [];
        const colorBoostEvents = [];
        const rotationEvents = [];
        const bpmEvents = [];
        const _events = data._events || [];
        for (let i = 0; i < _events.length; i++) {
            const obj = _events[i];
            switch (obj?._type) {
                case 5:
                    colorBoostEvents.push(colorBoostEvent.deserialize(obj));
                    break;
                case 14:
                case 15:
                    rotationEvents.push(rotationEvent.deserialize(obj));
                    break;
                case 100:
                    bpmEvents.push(bpmEvent.deserialize(obj));
                    break;
                default: {
                    const evt = basicEvent.deserialize(obj);
                    if (preV25 < 0) {
                        if (obj._type === 10) {
                            bpmEvents.push(bpmEvent.deserialize(obj));
                        }
                        else {
                            evt.floatValue = 1;
                            basicEvents.push(evt);
                        }
                    }
                    else {
                        basicEvents.push(evt);
                    }
                }
            }
        }
        return createBeatmap({
            version: 2,
            filename: options?.filename,
            lightshowFilename: options?.lightshowFilename,
            difficulty: createDifficulty({
                colorNotes,
                bombNotes,
                obstacles: data._obstacles?.map((x) => {
                    return obstacle.deserialize(x);
                }),
                rotationEvents,
                bpmEvents,
                customData: data._customData,
            }),
            lightshow: createLightshow({
                waypoints: data._waypoints?.map((x) => {
                    return waypoint.deserialize(x);
                }),
                basicEvents,
                colorBoostEvents,
                useNormalEventsAsCompatibleEvents: true,
                basicEventTypesWithKeywords: basicEventTypesWithKeywords.deserialize(data._specialEventsKeywordFilters ?? {}),
            }),
        });
    },
};
