/** Clamp value between min and max. */
export function clamp(value, min, max) {
    return Math.min(Math.max(min, value), max);
}
/**
 * Normalize value to 0-1 from given min and max value.
 *
 * Returns 1 if `min == max`, values beyond will be extrapolated.
 */
export function normalize(value, min, max) {
    if (min === max)
        return 1;
    return (value - min) / (max - min);
}
/**
 * Linear interpolate between start to end time given alpha value.
 * ```ts
 * const num = lerp(0.5, 4, 8); // returns 6
 * ```
 *
 * Alpha value must be in range of `0-1`, otherwise extrapolated.
 */
export function lerp(alpha, from, to) {
    return from + (to - from) * alpha;
}
/**
 * Returns alpha value from interpolated value.
 * ```ts
 * const num = invLerp(6, 4, 8); // returns 0.5
 * ```
 */
export function invLerp(value, from, to) {
    return (value - from) / (to - from);
}
/**
 * Remap the value from original range to target range.
 * ```ts
 * const num = remap(6, 4, 8, 40, 60); // returns 50
 * ```
 */
export function remap(value, origFrom, origTo, targetFrom, targetTo) {
    const alpha = invLerp(value, origFrom, origTo);
    return lerp(alpha, targetFrom, targetTo);
}
/** Returns true if both value are approximately equal within given tolerance. */
export function nearEqual(n1, n2, tolerance = Number.EPSILON) {
    return Math.abs(n1 - n2) <= tolerance;
}
/** Generate 32-bit hash with Java implementation. */
export function hashCode(str) {
    str = str.toString();
    let hash = 0;
    for (let i = 0; i < str.length; i++) {
        hash = (hash << 5) - hash + str.charCodeAt(i);
        hash |= 0;
    }
    return hash;
}
/** Return number tuple in order. */
export function rearrangeTuple(min, max, inverse) {
    if (!inverse && min > max) {
        return [max, min];
    }
    if (inverse && min < max) {
        return [max, min];
    }
    return [min, max];
}
/** Round number to given decimal places. */
export function round(num, d = 0) {
    const r = Math.pow(10, d);
    return Math.round(num * r) / r;
}
/** Sum numbers in an array. */
export function sumAry(nums) {
    let sum = 0;
    for (const n of nums)
        sum += n;
    return sum;
}
/** Product numbers in an array. */
export function productAry(nums) {
    if (!nums.length) {
        return 0;
    }
    let prod = 1;
    for (const n of nums)
        prod *= n;
    return prod;
}
/** Mean of numbers in an array. */
export function mean(nums) {
    if (!nums.length) {
        return 0;
    }
    return sumAry(nums) / nums.length;
}
/** Median of numbers in an array. */
export function median(nums) {
    if (!nums.length) {
        return 0;
    }
    const ary = nums.map((n) => n).sort((a, b) => a - b);
    const mid = Math.floor(ary.length / 2);
    if (ary.length % 2) {
        return ary[mid];
    }
    return (ary[mid - 1] + ary[mid]) / 2;
}
