import { hashCode, rearrangeTuple, round } from './helpers.js';
// Randomly generate seed if not provided.
const _seed = { ref: hashCode(Math.random()) };
/**
 * Mulberry32 algorithm.
 *
 * shamelessly taken from stackoverflow
 */
function _pRandom(seed) {
    const _s = typeof seed === 'number' ? { ref: seed } : seed;
    return function () {
        let s = (_s.ref += 0x6d2b79f5);
        s = Math.imul(s ^ (s >>> 15), s | 1);
        s ^= s + Math.imul(s ^ (s >>> 7), s | 61);
        return ((s ^ (s >>> 14)) >>> 0) / 4294967296;
    };
}
const _instPRandom = _pRandom(_seed);
function _random(min, max, rounding = false, fn = Math.random) {
    if (typeof min === 'boolean' || (!min && typeof min !== 'number')) {
        if (min) {
            return Math.round(fn());
        }
        return fn();
    }
    if (typeof max === 'boolean' || (!max && typeof max !== 'number')) {
        let result = fn() * min;
        if (max) {
            result = Math.round(result);
        }
        return result;
    }
    [min, max] = rearrangeTuple(min, max);
    const result = fn() * (max - min) + min;
    return rounding
        ? round(result, typeof rounding === 'number' && rounding > 0 ? rounding : 0)
        : result;
}
export function pRandom(min, max, rounding = false) {
    return _random(min, max, rounding, _instPRandom);
}
/**
 * Create instance of pseudorandom function.
 * ```ts
 * const pRandom = utils.pRandomFn('seed');
 * console.log(pRandom());
 * ```
 *
 * **NOTE:** Seed cannot be reset.
 */
export function pRandomFn(seed = Math.random()) {
    const _seed = hashCode(seed);
    const _func = _pRandom(_seed);
    return function (min, max, rounding = false) {
        return _random(min, max, rounding, _func);
    };
}
/**
 * Set seed for pseudorandom generator.
 *
 * Recalling this resets the seed.
 *
 * If this is never called, defaults to randomly generated seed.
 */
export function pRandomSeed(seed) {
    _seed.ref = hashCode(seed);
}
export function random(min, max, rounding = false) {
    return _random(min, max, rounding, Math.random);
}
/**
 * JS implementation of gaussian random using Box-Muller transformation and skew-normal.
 *
 * This returns value closer to `[-6, 6]` range but can be completely outside of the range due to precision.
 *
 * @link https://spin.atomicobject.com/skew-normal-prng-javascript/
 */
export function randomNormal(mean = 0, stdev = 1, skew = 0, rand = Math.random) {
    let u1 = 0;
    let u2 = 0;
    while (u1 === 0)
        u1 = rand();
    while (u2 === 0)
        u2 = rand();
    const mag = Math.sqrt(-2.0 * Math.log(u1));
    const dir = 2.0 * Math.PI * u2;
    const u0 = mag * Math.cos(dir);
    if (!skew)
        return mean + stdev * u0;
    const v = mag * Math.sin(dir);
    const co = skew / Math.sqrt(1 + skew * skew);
    const u3 = co * u0 + Math.sqrt(1 - co * co) * v;
    const z = u0 >= 0 ? u3 : -u3;
    return mean + stdev * z;
}
