import { lerp, nearEqual } from './helpers.js';
/** Check if value is `Vector2` */
export function isVector2(obj) {
    return (Array.isArray(obj) &&
        obj.length === 2 &&
        obj.every((n) => typeof n === 'number'));
}
/** Check if value is `Vector3` */
export function isVector3(obj) {
    return (Array.isArray(obj) &&
        obj.length === 3 &&
        obj.every((n) => typeof n === 'number'));
}
/** Check if value is `Vector4` */
export function isVector4(obj) {
    return (Array.isArray(obj) &&
        obj.length === 4 &&
        obj.every((n) => typeof n === 'number'));
}
function vectorImpl(operation) {
    return function (vec, value, ...args) {
        if (!vec)
            return vec;
        if (typeof value === 'number') {
            return vec.map((v) => operation(v, value, ...args));
        }
        vec = [...vec];
        if (value) {
            if (Array.isArray(value)) {
                for (let i = 0; i < vec.length; i++) {
                    if (typeof value[i] === 'number') {
                        vec[i] = operation(vec[i], value[i], ...args);
                    }
                }
            }
            else {
                switch (vec.length) {
                    case 4: {
                        vec[3] = operation(vec[3], value.w ?? 0, ...args);
                    }
                    /* falls through */
                    case 3: {
                        vec[2] = operation(vec[2], value.z ?? 0, ...args);
                    }
                    /* falls through */
                    case 2: {
                        vec[1] = operation(vec[1], value.y ?? 0, ...args);
                        vec[0] = operation(vec[0], value.x ?? 0, ...args);
                    }
                }
            }
        }
        return vec;
    };
}
export function vectorAdd(vec, value) {
    return vectorImpl((x, n) => x + n)(vec, value);
}
export function vectorSub(vec, value) {
    return vectorImpl((x, n) => x - n)(vec, value);
}
export function vectorMul(vec, value) {
    return vectorImpl((x, n) => (n ? x * n : x))(vec, value);
}
export function vectorDiv(vec, value) {
    return vectorImpl((x, n) => (n ? x / n : x))(vec, value);
}
export function lerpVector(alpha, vec, value) {
    return vectorImpl((x, n, alpha) => lerp(alpha, x, n))(vec, value, alpha);
}
export function vectorDistance(v1, v2) {
    const diff = vectorSub(v1, v2);
    let acc = 0;
    for (let i = 0; i < diff.length; i++) {
        const x = diff[i] ** 2;
        acc += x;
    }
    return Math.sqrt(acc);
}
export function vectorIsVertical(v1, v2, epsilon = 0.001) {
    const [dX] = vectorSub(v1, v2);
    return Math.abs(dX) <= epsilon;
}
export function vectorIsHorizontal(v1, v2, epsilon = 0.001) {
    const [, dY] = vectorSub(v1, v2);
    return Math.abs(dY) <= epsilon;
}
export function vectorIsDiagonal(v1, v2, epsilon = 0.001) {
    const [dX, dY] = vectorSub(v1, v2);
    return nearEqual(Math.abs(dX), Math.abs(dY), epsilon);
}
export function vectorIsInline(v1, v2, lapping = 0.5) {
    const distance = vectorDistance(v1, v2);
    return distance <= lapping;
}
export function vectorIsAdjacent(v1, v2, epsilon = 0.001) {
    const distance = vectorDistance(v1, v2);
    return distance > 0.5 - epsilon && distance < 1 + epsilon;
}
export function vectorIsWindow(v1, v2, gap = 1.8) {
    const distance = vectorDistance(v1, v2);
    return distance > gap;
}
export function vectorIsSlantedWindow(v1, v2) {
    return (vectorIsWindow(v1, v2, 1.8) &&
        !vectorIsDiagonal(v1, v2, 0.001) &&
        !vectorIsHorizontal(v1, v2, 0.001) &&
        !vectorIsVertical(v1, v2, 0.001));
}
export function vectorMagnitude(vec) {
    let sum = 0;
    for (let i = 0; i < vec.length; i++) {
        sum += vec[i] * vec[i];
    }
    return Math.sqrt(sum);
}
export function vectorNormalize(vec) {
    return vectorDiv(vec, vectorMagnitude(vec));
}
export function vectorEqual(v1, v2) {
    if (v1.length !== v2.length)
        return false;
    for (let i = 0; i < v1.length; i++) {
        if (v1[i] !== v2[i])
            return false;
    }
    return true;
}
export function vectorNearEqual(v1, v2, tolerance = 0.001) {
    if (v1.length !== v2.length)
        return false;
    for (let i = 0; i < v1.length; i++) {
        if (!nearEqual(v1[i], v2[i], tolerance))
            return false;
    }
    return true;
}
