import type { GetPositionFn, MirrorFn } from '../../../types/beatmap/shared/functions.js';
import type { IWrapGridObject } from '../../../types/beatmap/wrapper/gridObject.js';
import type { Vector2 } from '../../../types/vector.js';
import { BaseObject } from './baseObject.js';
/**
 * Base grid beatmap object.
 *
 * This appear within beatmap lane.
 *
 * @abstract
 */
export declare abstract class GridObject extends BaseObject implements IWrapGridObject {
    posX: IWrapGridObject['posX'];
    posY: IWrapGridObject['posY'];
    laneRotation: IWrapGridObject['laneRotation'];
    setPosX(value: this['posX']): this;
    setPosY(value: this['posY']): this;
    setLaneRotation(value: this['laneRotation']): this;
    /**
     * Mirror a grid object, apply alternative flip and Noodle Extensions if available.
     * ```ts
     * obj.mirror(false, optionalFn);
     * ```
     *
     * Alternative flip is true by default. This implementation is typically used to flip color of note.
     */
    mirror(_flipAlt?: boolean, fn?: MirrorFn<this>): this;
    /**
     * Get object position and return the Beatwalls' position x and y value in tuple.
     * ```ts
     * const objPos = obj.getPosition(optionalFn);
     * ```
     */
    getPosition(fn?: GetPositionFn<this>): Vector2;
    /**
     * Get two objects and return the distance between two objects.
     * ```ts
     * if (obj.getDistance(objCompare, optionalFn)) {}
     * ```
     */
    getDistance(compareTo: typeof this, fn?: GetPositionFn<this>): number;
    getDistance(compareTo: IWrapGridObject, fn?: GetPositionFn<IWrapGridObject>): number;
    /**
     * Compare two objects and return if the objects is in vertical alignment.
     * ```ts
     * if (obj.isVertical(objCompare, optionalFn)) {}
     * ```
     */
    isVertical(compareTo: typeof this, fn?: GetPositionFn<this>): boolean;
    isVertical(compareTo: IWrapGridObject, fn?: GetPositionFn<IWrapGridObject>): boolean;
    /**
     * Compare two objects and return if the objects is in horizontal alignment.
     * ```ts
     * if (obj.isHorizontal(objCompare, optionalFn)) {}
     * ```
     */
    isHorizontal(compareTo: typeof this, fn?: GetPositionFn<this>): boolean;
    isHorizontal(compareTo: IWrapGridObject, fn?: GetPositionFn<IWrapGridObject>): boolean;
    /**
     * Compare two objects and return if the objects is in diagonal alignment.
     * ```ts
     * if (obj.isDiagonal(objCompare, optionalFn)) {}
     * ```
     */
    isDiagonal(compareTo: typeof this, fn?: GetPositionFn<this>): boolean;
    isDiagonal(compareTo: IWrapGridObject, fn?: GetPositionFn<IWrapGridObject>): boolean;
    /**
     * Compare two  objects and return if the  objects is an inline.
     * ```ts
     * if (obj.isInline(objCompare, optionalLapping, optionalFn)) {}
     * ```
     */
    isInline(compareTo: typeof this, lapping?: number | null, fn?: GetPositionFn<this>): boolean;
    isInline(compareTo: IWrapGridObject, lapping?: number | null, fn?: GetPositionFn<IWrapGridObject>): boolean;
    /**
     * Compare two objects and return if the objects is adjacent.
     * ```ts
     * if (obj.isAdjacent(objCompare, optionalFn)) {}
     * ```
     */
    isAdjacent(compareTo: typeof this, fn?: GetPositionFn<this>): boolean;
    isAdjacent(compareTo: IWrapGridObject, fn?: GetPositionFn<IWrapGridObject>): boolean;
    /**
     * Compare two objects and return if the objects is a window.
     * ```ts
     * if (obj.isWindow(objCompare, optionalDistance, optionalFn)) {}
     * ```
     */
    isWindow(compareTo: typeof this, distance?: number | null, fn?: GetPositionFn<this>): boolean;
    isWindow(compareTo: IWrapGridObject, distance?: number | null, fn?: GetPositionFn<IWrapGridObject>): boolean;
    /**
     * Compare two objects and return if the objects is a slanted window.
     * ```ts
     * if (obj.isSlantedWindow(objCompare, distance, optionalFn)) {}
     * ```
     */
    isSlantedWindow(compareTo: typeof this, fn?: GetPositionFn<this>): boolean;
    isSlantedWindow(compareTo: IWrapGridObject, fn?: GetPositionFn<IWrapGridObject>): boolean;
}
//# sourceMappingURL=gridObject.d.ts.map