"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AudioData = void 0;
exports.createAudioData = createAudioData;
const json_js_1 = require("../../utils/misc/json.js");
const baseItem_js_1 = require("./abstract/baseItem.js");
function createAudioData(data = {}) {
    return {
        version: data.version ?? -1,
        filename: data.filename ?? 'AudioData.dat',
        audioChecksum: data.audioChecksum ?? '',
        sampleCount: data.sampleCount ?? 44100,
        frequency: data.frequency ?? 0,
        bpmData: data.bpmData?.map((e) => ({
            startSampleIndex: e.startSampleIndex ?? 0,
            endSampleIndex: e.endSampleIndex ?? 0,
            startBeat: e.startBeat ?? 0,
            endBeat: e.endBeat ?? 0,
        })) ?? [],
        lufsData: data.lufsData?.map((e) => ({
            startSampleIndex: e.startSampleIndex ?? 0,
            endSampleIndex: e.endSampleIndex ?? 0,
            lufs: e.lufs ?? 0,
        })) ?? [],
        customData: (0, json_js_1.deepCopy)({ ...data.customData }),
    };
}
/**
 * Core beatmap audio data.
 *
 * This object is writable into file.
 */
class AudioData extends baseItem_js_1.BaseItem {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "version", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "filename", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "audioChecksum", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "sampleCount", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "frequency", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "bpmData", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lufsData", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.version = data.version ?? AudioData.defaultValue.version;
        this.filename = data.filename ?? AudioData.defaultValue.filename;
        this.audioChecksum = data.audioChecksum ?? AudioData.defaultValue.audioChecksum;
        this.sampleCount = data.sampleCount ?? AudioData.defaultValue.sampleCount;
        this.frequency = data.frequency ?? AudioData.defaultValue.frequency;
        this.bpmData = (data.bpmData ?? AudioData.defaultValue.bpmData).map((e) => ({
            startBeat: e?.startBeat || 0,
            startSampleIndex: e?.startSampleIndex || 0,
            endBeat: e?.endBeat || 0,
            endSampleIndex: e?.endSampleIndex || 0,
        }));
        this.lufsData = (data.lufsData ?? AudioData.defaultValue.lufsData).map((e) => ({
            startSampleIndex: e?.startSampleIndex || 0,
            endSampleIndex: e?.endSampleIndex || 0,
            lufs: e?.lufs || 0,
        }));
        this.customData = (0, json_js_1.deepCopy)(data.customData ?? AudioData.defaultValue.customData);
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            this.frequency >= 0 &&
            this.sampleCount >= 0 &&
            this.bpmData.every((bpm) => bpm.endBeat > bpm.startBeat && bpm.endSampleIndex > bpm.startSampleIndex);
    }
    setFilename(filename) {
        this.filename = filename;
        return this;
    }
    setVersion(version) {
        this.version = version;
        return this;
    }
    setSampleCount(value) {
        this.sampleCount = value;
        return this;
    }
    setFrequency(value) {
        this.frequency = value;
        return this;
    }
    fromBpmEvents(data, frequency = 44100, sampleCount) {
        if (!data.length)
            return this;
        this.frequency = frequency;
        if (sampleCount)
            this.sampleCount = sampleCount;
        this.bpmData = [];
        for (let i = 0; i < data.length - 1; i++) {
            const currentBpmEvent = data[i];
            const nextBpmEvent = data[i + 1];
            this.bpmData.push({
                startSampleIndex: Math.floor(currentBpmEvent.time * this.frequency),
                endSampleIndex: Math.floor(nextBpmEvent.time * this.frequency),
                startBeat: currentBpmEvent.time,
                endBeat: nextBpmEvent.time,
            });
        }
        const lastBpmEvent = data[data.length - 1];
        const lastStartSampleIndex = Math.floor(lastBpmEvent.time * this.frequency);
        const secondsDiff = (this.sampleCount - lastStartSampleIndex) / this.frequency;
        const jsonBeatsDiff = secondsDiff * (lastBpmEvent.bpm / 60);
        this.bpmData.push({
            startSampleIndex: lastStartSampleIndex,
            endSampleIndex: this.sampleCount,
            startBeat: lastBpmEvent.time,
            endBeat: lastBpmEvent.time + jsonBeatsDiff,
        });
        return this;
    }
    getBpmEvents() {
        return this.bpmData.map((bd) => {
            const sample = bd.endSampleIndex - bd.startSampleIndex;
            const bpm = ((bd.endBeat - bd.startBeat) / (sample / this.frequency)) * 60;
            return {
                time: bd.startBeat,
                bpm,
                customData: {},
            };
        });
    }
    sort() {
        this.bpmData.sort((a, b) => a.startBeat - b.startBeat);
        return this;
    }
}
exports.AudioData = AudioData;
Object.defineProperty(AudioData, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createAudioData()
});
