"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BasicEvent = void 0;
exports.createBasicEvent = createBasicEvent;
const json_js_1 = require("../../utils/misc/json.js");
const basicEvent_js_1 = require("../helpers/core/basicEvent.js");
const baseObject_js_1 = require("./abstract/baseObject.js");
function createBasicEvent(data = {}) {
    return {
        time: data.time ?? 0,
        type: data.type ?? 0,
        value: data.value ?? 0,
        floatValue: data.floatValue ?? 0,
        customData: (0, json_js_1.deepCopy)({ ...data.customData }),
    };
}
/**
 * Core beatmap basic event.
 */
class BasicEvent extends baseObject_js_1.BaseObject {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "type", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "value", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "floatValue", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.time = data.time ?? BasicEvent.defaultValue.time;
        this.type = data.type ?? BasicEvent.defaultValue.type;
        this.value = data.value ?? BasicEvent.defaultValue.value;
        this.floatValue = data.floatValue ?? BasicEvent.defaultValue.floatValue;
        this.customData = (0, json_js_1.deepCopy)(data.customData ?? BasicEvent.defaultValue.customData);
    }
    /**
     * Check if event is a valid type.
     * ```ts
     * if (event.isValidType()) {}
     * ```
     */
    isValidType() {
        return (0, basicEvent_js_1.isValidEventType)(this.type);
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            this.isValidType() &&
            this.value >= 0 &&
            !(!this.isLaserRotationEvent() && this.value > 12 && !this.isOldChroma());
    }
    setType(value) {
        this.type = value;
        return this;
    }
    setValue(value) {
        this.value = value;
        return this;
    }
    setFloatValue(value) {
        this.floatValue = value;
        return this;
    }
    /**
     * Check if light event is an off event.
     * ```ts
     * if (event.isOff()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isOff() {
        return (0, basicEvent_js_1.isOffEventValue)(this.value);
    }
    /**
     * Check if light event is an on event.
     * ```ts
     * if (event.isOn()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isOn() {
        return (0, basicEvent_js_1.isOnEventValue)(this.value);
    }
    /**
     * Check if light event is a flash event.
     * ```ts
     * if (event.isFlash()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isFlash() {
        return (0, basicEvent_js_1.isFlashEventValue)(this.value);
    }
    /**
     * Check if light event is a fade event.
     * ```ts
     * if (event.isFade()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isFade() {
        return (0, basicEvent_js_1.isFadeEventValue)(this.value);
    }
    /**
     * Check if light event is a transition event.
     * ```ts
     * if (event.isTransition()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isTransition() {
        return (0, basicEvent_js_1.isTransitionEventValue)(this.value);
    }
    /**
     * Check if light event is a blue light.
     * ```ts
     * if (event.isBlue()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isBlue() {
        return (0, basicEvent_js_1.isBlueEventValue)(this.value);
    }
    /**
     * Check if light event is a red light.
     * ```ts
     * if (event.isRed()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isRed() {
        return (0, basicEvent_js_1.isRedEventValue)(this.value);
    }
    /**
     * Check if light event is a white light.
     * ```ts
     * if (event.isWhite()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isWhite() {
        return (0, basicEvent_js_1.isWhiteEventValue)(this.value);
    }
    /**
     * Check if event is a light event.
     * ```ts
     * if (event.isLightEvent()) {}
     * ```
     */
    isLightEvent(environment) {
        return (0, basicEvent_js_1.isLightEventType)(this.type, environment);
    }
    /**
     * Check if event is a boost event.
     * ```ts
     * if (event.isColorBoost()) {}
     * ```
     */
    isColorBoost() {
        return (0, basicEvent_js_1.isColorBoostEventType)(this.type);
    }
    /**
     * Check if event is a ring event.
     * ```ts
     * if (event.isRingEvent()) {}
     * ```
     *
     * This does not check for ring zoom.
     */
    isRingEvent(environment) {
        return (0, basicEvent_js_1.isRingEventType)(this.type, environment);
    }
    /**
     * Check if event is a laser rotation event.
     * ```ts
     * if (event.isLaserRotationEvent()) {}
     * ```
     */
    isLaserRotationEvent(environment) {
        return (0, basicEvent_js_1.isLaserRotationEventType)(this.type, environment);
    }
    /**
     * Check if event is a lane rotation event.
     * ```ts
     * if (event.isLaneRotationEvent()) {}
     * ```
     */
    isLaneRotationEvent() {
        return (0, basicEvent_js_1.isLaneRotationEventType)(this.type);
    }
    /**
     * Check if event is a extra event.
     * ```ts
     * if (event.isExtraEvent()) {}
     * ```
     */
    isExtraEvent(environment) {
        return (0, basicEvent_js_1.isExtraEventType)(this.type, environment);
    }
    /**
     * Check if event is a special event.
     * ```ts
     * if (event.isSpecialEvent()) {}
     * ```
     */
    isSpecialEvent(environment) {
        return (0, basicEvent_js_1.isSpecialEventType)(this.type, environment);
    }
    /**
     * Check if event is a BPM change event.
     * ```ts
     * if (event.isBpmEvent()) {}
     * ```
     */
    isBpmEvent() {
        return (0, basicEvent_js_1.isBpmChangeEventType)(this.type);
    }
    isNjsEvent() {
        return (0, basicEvent_js_1.isNjsChangeEventType)(this.type);
    }
    /**
     * Not to be confused with `isLightEvent`, this checks for event that affects the environment/lighting.
     * ```ts
     * if (event.isLightingEvent()) {}
     * ```
     */
    isLightingEvent() {
        return (0, basicEvent_js_1.isLightingEventType)(this.type);
    }
    /**
     * Check if event has old Chroma properties.
     * ```ts
     * if (event.isOldChroma()) {}
     * ```
     */
    isOldChroma() {
        return (0, basicEvent_js_1.isOldChromaEventValue)(this.value);
    }
}
exports.BasicEvent = BasicEvent;
Object.defineProperty(BasicEvent, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createBasicEvent()
});
