"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculateNps = calculateNps;
exports.calculateNpsPeak = calculateNpsPeak;
exports.getFirstInteractiveTime = getFirstInteractiveTime;
exports.getLastInteractiveTime = getLastInteractiveTime;
exports.findFirstInteractiveObstacleTime = findFirstInteractiveObstacleTime;
exports.findLastInteractiveObstacleTime = findLastInteractiveObstacleTime;
const obstacle_js_1 = require("./core/obstacle.js");
const sort_js_1 = require("./sort.js");
/** Calculates the number of notes per second. */
function calculateNps(beatmap, duration) {
    const notes = beatmap.difficulty.colorNotes;
    return duration ? notes.length / duration : 0;
}
/** Calculates the peak number of notes per second given beat count. */
function calculateNpsPeak(beatmap, beatCount, timeProcessor) {
    let peakNPS = 0;
    let currentSectionStart = 0;
    const notes = beatmap.difficulty.colorNotes;
    for (let i = 0; i < notes.length; i++) {
        while (notes[i].time - notes[currentSectionStart].time > beatCount) {
            currentSectionStart++;
        }
        peakNPS = Math.max(peakNPS, (i - currentSectionStart + 1) / timeProcessor.toRealTime(beatCount));
    }
    return peakNPS;
}
/**
 * Get the first interactive time.
 *
 * Color notes, bomb notes, chains and obstacles in middle lane are considered interactive object.
 */
function getFirstInteractiveTime(beatmap) {
    const notes = [
        ...beatmap.difficulty.colorNotes,
        ...beatmap.difficulty.bombNotes,
        ...beatmap.difficulty.chains,
    ].sort(sort_js_1.sortNoteFn);
    let firstNoteTime = Number.MAX_VALUE;
    if (notes.length > 0) {
        firstNoteTime = notes[0].time;
    }
    const firstInteractiveObstacleTime = findFirstInteractiveObstacleTime(beatmap);
    return Math.min(firstNoteTime, firstInteractiveObstacleTime);
}
/**
 * Get the last interactive time.
 *
 * Color notes, bomb notes, chains and obstacles in middle lane are considered interactive object.
 */
function getLastInteractiveTime(beatmap) {
    const notes = [
        ...beatmap.difficulty.colorNotes,
        ...beatmap.difficulty.bombNotes,
        ...beatmap.difficulty.chains,
    ].sort(sort_js_1.sortNoteFn);
    let lastNoteTime = 0;
    if (notes.length > 0) {
        lastNoteTime = notes[notes.length - 1].time;
    }
    const lastInteractiveObstacleTime = findLastInteractiveObstacleTime(beatmap);
    return Math.max(lastNoteTime, lastInteractiveObstacleTime);
}
/**
 * Get the first obstacle interactive time.
 */
function findFirstInteractiveObstacleTime(beatmap) {
    for (let i = 0, len = beatmap.difficulty.obstacles.length; i < len; i++) {
        if ((0, obstacle_js_1.isInteractiveObstacle)(beatmap.difficulty.obstacles[i])) {
            return beatmap.difficulty.obstacles[i].time;
        }
    }
    return Number.MAX_VALUE;
}
/**
 * Get the last obstacle interactive time.
 */
function findLastInteractiveObstacleTime(beatmap) {
    let obstacleEnd = 0;
    for (let i = beatmap.difficulty.obstacles.length - 1; i >= 0; i--) {
        if ((0, obstacle_js_1.isInteractiveObstacle)(beatmap.difficulty.obstacles[i])) {
            obstacleEnd = Math.max(obstacleEnd, beatmap.difficulty.obstacles[i].time + beatmap.difficulty.obstacles[i].duration);
        }
    }
    return obstacleEnd;
}
