"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.remapDedupe = remapDedupe;
exports.purgeZeros = purgeZeros;
exports.deepClean = deepClean;
const logger_js_1 = require("../../logger.js");
const helpers_js_1 = require("../../utils/math/helpers.js");
function tag(name) {
    return ['helpers', name];
}
/**
 * Remap and deduplicate data in array.
 *
 * @param data The data array to remap and deduplicate.
 * @returns The remapped and deduplicated data and a map of the original index to the new index.
 */
function remapDedupe(data) {
    const newData = [];
    const remapIdx = new Map();
    for (let i = 0; i < data.length; i++) {
        const str = JSON.stringify(data[i]);
        let idx = newData.indexOf(str);
        if (idx === -1) {
            idx = newData.length;
            newData.push(str);
        }
        remapIdx.set(i, idx);
    }
    return [newData.map((d) => JSON.parse(d)), remapIdx];
}
/**
 * Delete props with zero values.
 *
 * @param data The data to purge props.
 */
// deno-lint-ignore no-explicit-any
function purgeZeros(data) {
    for (const k in data) {
        if ((typeof data[k] === 'number' || typeof data[k] === 'boolean') && !data[k]) {
            delete data[k];
        }
    }
}
/** Recursively clean object data. */
function deepClean(
// deno-lint-ignore no-explicit-any
obj, name, options) {
    for (const k in obj) {
        const d = obj[k];
        if (typeof d === 'number' && options.floatTrim) {
            obj[k] = (0, helpers_js_1.round)(d, options.floatTrim);
            continue;
        }
        if (typeof d === 'string' && options.stringTrim) {
            obj[k] = d.trim();
            continue;
        }
        if (typeof obj[k] === 'boolean') {
            continue;
        }
        // throw or default null to 0
        if (d === null) {
            if (options.throwNullish) {
                throw new Error(`null value found in object key ${name}.${k}.}`);
            }
            else {
                if (Array.isArray(obj)) {
                    logger_js_1.logger.tError(tag('deepClean'), `null value found in array ${name}[${k}], defaulting to 0...`);
                    obj[k] = 0;
                }
                else {
                    logger_js_1.logger.tError(tag('deepClean'), `null value found in object key ${name}.${k}, deleting...`);
                    delete obj[k];
                }
            }
            continue;
        }
        // recursion stuff
        if (typeof d === 'object') {
            // filter out undefined in array
            if (Array.isArray(d)) {
                const len = d.length;
                const newAry = d.filter((e) => e !== undefined);
                if (len !== newAry.length) {
                    if (options.throwNullish) {
                        throw new Error(`undefined found in array key ${name}.${k}.}`);
                    }
                    else {
                        logger_js_1.logger.tError(tag('deepClean'), `undefined found in array key ${name}.${k}, replacing array with no undefined...`);
                        obj[k] = newAry;
                    }
                }
            }
            deepClean(
            // deno-lint-ignore ban-types
            d, Array.isArray(obj) ? `${name}[${k}]` : `${name}.${k}`, options);
        }
        // remove unnecessary empty array/object property if exist and not part of schema
        if ((Array.isArray(d) && !d.length) || JSON.stringify(d) === '{}') {
            delete obj[k];
            continue;
        }
    }
}
