"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.field = field;
exports.entity = entity;
exports.mask = mask;
// deno-lint-ignore-file no-explicit-any
const _deps_js_1 = require("../../_deps.js");
const logger_js_1 = require("../../logger.js");
const json_js_1 = require("../../utils/misc/json.js");
const version_js_1 = require("../helpers/version.js");
/** Helper function to augment a schema with the necessary context for validation within a top-level entity schema. */
function field(schema, options) {
    const [base, ...rest] = 'pipe' in schema ? schema.pipe : [schema];
    // hack: because valibot does not support preprocessing, it will assume all keys should be present and validated regardless of its supported version(s).
    // we need to optionalize the provided schema to bypass this quirk and allow the version checks to run without failing early
    return _deps_js_1.v.pipe(_deps_js_1.v.optional(base), ...rest, _deps_js_1.v.metadata({ version: options?.version }));
}
function checkVersion(schema, { version, dataset, addIssue, }) {
    const [base, ...pipeline] = 'pipe' in schema ? schema.pipe : [schema];
    let unwrapped = base;
    // unwrap the schema from its optionalized parent
    if ('wrapped' in unwrapped) {
        unwrapped = _deps_js_1.v.unwrap(unwrapped);
    }
    // extract the metadata from the pipeline to get the required context for versioning checks
    const ctx = pipeline.find((x) => x.kind === 'metadata');
    logger_js_1.logger.tDebug(['schema', 'checkVersion'], `for ${unwrapped.type}:\n  schema version: \t${ctx?.metadata.version ?? 'undefined'}\n  dataset version: \t${version}`);
    const input = dataset.value;
    if (ctx && ctx.metadata.version) {
        // if metadata is present, check for version mismatches on existing fields
        const { version: schemaVersion } = ctx.metadata;
        const comparator = (0, version_js_1.compareVersion)(version, schemaVersion);
        // if the field has an unsupported version and a value is present...
        if (comparator < 0 && input !== undefined) {
            return addIssue({
                message: 'Mismatched version for field',
                input: input,
                received: version,
                expected: schemaVersion,
            });
        }
        // if the field has a supported version and a value is missing for the field...
        const isOptionalizedSchema = ['optional', 'undefinedable'].includes(unwrapped.type);
        if (comparator >= 0 && input === undefined && !isOptionalizedSchema) {
            return addIssue({
                message: 'Missing required value for versioned field',
                input: input,
                received: typeof input,
                expected: unwrapped.type,
            });
        }
    }
    else {
        // if metadata is not present, skip the version checks entirely and run the original validation flow
        const { issues } = _deps_js_1.v.safeParse(unwrapped, dataset.value);
        for (const issue of issues ?? []) {
            return addIssue(issue);
        }
    }
    // for array data, cascade checks to all items
    if (_deps_js_1.v.isOfType('array', unwrapped) && Array.isArray(input)) {
        const schema = unwrapped.item;
        for (let i = 0; i < input.length; i++) {
            const value = input[i];
            checkVersion(schema, {
                version,
                addIssue: (info) => {
                    const path = {
                        type: 'array',
                        origin: 'value',
                        input: input,
                        key: i,
                        value: value,
                    };
                    return addIssue({
                        ...info,
                        path: [path, ...(info?.path ?? [])],
                    });
                },
                dataset: { ...dataset, value: value },
            });
        }
    }
    // for object data, cascade checks to all key/value entries
    if (_deps_js_1.v.isOfType('object', unwrapped) && (0, json_js_1.isRecord)(input)) {
        const entries = unwrapped.entries;
        for (const key in entries) {
            const value = input[key];
            checkVersion(entries[key], {
                version,
                addIssue: (info) => {
                    const path = {
                        type: 'object',
                        origin: 'value',
                        input: input,
                        key: key,
                        value: value,
                    };
                    return addIssue({
                        ...info,
                        path: [path, ...(info?.path ?? [])],
                    });
                },
                dataset: { ...dataset, value: value },
            });
        }
    }
}
/** Helper function to create an "entity" (object-like) schema, which recursively performs version checks on all nested entries. */
function entity(resolveVersion, entries) {
    return _deps_js_1.v.pipe(
    // we assume no additional fields are present other than what is supported by the schema.
    // if we have unknown entries, we'll simply omit them from the validation output and pass validation as normal.
    // that way, future updates that introduce new fields won't break existing validation flows.
    _deps_js_1.v.object(entries), _deps_js_1.v.rawCheck(({ dataset, addIssue }) => {
        if (!dataset.typed)
            return;
        // pull the entity version directly from the entity data using a resolver
        const version = resolveVersion(dataset.value);
        // run version checks for all key/value entries defined within the schema
        for (const key in entries) {
            const value = dataset.value[key];
            checkVersion(entries[key], {
                version,
                dataset: { ...dataset, value },
                addIssue: (info) => {
                    const path = {
                        type: 'object',
                        origin: 'value',
                        input: dataset.value,
                        key: key,
                        value: value,
                    };
                    return addIssue({
                        ...info,
                        path: [path, ...(info?.path ?? [])],
                    });
                },
            });
        }
    }));
}
/** Helper function to cast the inferred input of a schema to a different type. */
function mask(schema) {
    return schema;
}
