"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LightMapper = void 0;
const basicEvent_js_1 = require("../../../beatmap/helpers/core/basicEvent.js");
const colorScheme_js_1 = require("../../../beatmap/shared/colorScheme.js");
const convertor_js_1 = require("../../../utils/colors/convertor.js");
const easings_js_1 = require("../../../utils/math/easings.js");
const json_js_1 = require("../../../utils/misc/json.js");
const lightID_js_1 = require("./lightID.js");
/**
 * This uses new lighting lighting syntax for v2 lighting including support for color and easing.
 * ```ts
 * const lightMapper = new LightMapper('FitbeatEnvironment');
 * lightMapper.light(8, 2, [eventBox]);
 * lightMapper.process(mapData);
 * ```
 * Not all functionality works here however, there are quirks that may not work too well.
 */
class LightMapper {
    constructor(environment) {
        Object.defineProperty(this, "lightIDMapping", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "environment", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "queue", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "events", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "boosts", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        this.lightIDMapping = (0, json_js_1.deepCopy)(lightID_js_1.LightIDList[environment], true);
        this.environment = environment;
    }
    light(time, type, eventBox, lightID) {
        this.queue.push({
            time,
            type,
            lightID: lightID ? lightID : this.lightIDMapping[type],
            eventBox: eventBox.map((eb) => {
                return {
                    indexFilter: eb.indexFilter
                        ? eb.indexFilter.type === 'Step and Offset'
                            ? {
                                type: 'Step and Offset',
                                id: eb.indexFilter.id ?? 0,
                                step: eb.indexFilter.step ?? 1,
                                reverse: eb.indexFilter.reverse ?? false,
                            }
                            : {
                                type: 'Division',
                                divide: eb.indexFilter.divide ??
                                    1,
                                id: eb.indexFilter.id ?? 0,
                                reverse: eb.indexFilter.reverse ?? false,
                            }
                        : {
                            type: 'Division',
                            divide: 1,
                            id: 0,
                            reverse: false,
                        },
                    beatDistribution: eb.beatDistribution ?? 0,
                    beatDistributionType: eb.beatDistributionType ?? 'Division',
                    beatDistributionEasing: eb.beatDistributionEasing ?? 'easeLinear',
                    brightnessDistribution: eb.brightnessDistribution ?? 0,
                    brightnessDistributionType: eb.brightnessDistributionType ?? 'Division',
                    brightnessDistributionEasing: eb.brightnessDistributionEasing ?? 'easeLinear',
                    hueDistribution: eb.hueDistribution ?? 0,
                    hueDistributionType: eb.hueDistributionType ?? 'Division',
                    hueDistributionEasing: eb.hueDistributionEasing ?? 'easeLinear',
                    affectFirst: eb.affectFirst ?? false,
                    events: eb.events
                        ? eb.events.map((ev) => {
                            return {
                                time: ev.time ?? 0,
                                color: ev.color ?? 0,
                                transition: ev.transition ?? 0,
                                brightness: ev.brightness ?? 1,
                                frequency: Math.max(ev.frequency ?? 0, 0),
                                customData: ev.customData ?? {},
                            };
                        })
                        : [],
                };
            }),
        });
        return this;
    }
    ring(time, type, customData) {
        this.events.push({ time, type, value: 0, floatValue: 0, customData: customData ?? {} });
        return this;
    }
    zoom(time, customData) {
        this.events.push({ time, type: 9, value: 0, floatValue: 0, customData: customData ?? {} });
        return this;
    }
    boost(time, toggle) {
        this.boosts.push({ time, toggle: toggle, customData: {} });
        return this;
    }
    internalEventValue(color, transition) {
        const transitionValue = {
            0: 0,
            1: 3,
            3: 1,
            4: 2,
        };
        return 1 + color * 4 + transitionValue[transition];
    }
    process(mapData, overwrite = true) {
        const events = [...this.events];
        this.queue.sort((a, b) => a.time - b.time);
        for (const q of this.queue) {
            q.eventBox.forEach((eb) => {
                let lid = [...q.lightID];
                if (eb.indexFilter.reverse) {
                    lid.reverse();
                }
                if (eb.indexFilter.type === 'Division') {
                    if (eb.indexFilter.divide > 1) {
                        lid = lid.slice(Math.floor(lid.length / eb.indexFilter.divide) *
                            eb.indexFilter.id, Math.ceil(lid.length / eb.indexFilter.divide) *
                            (eb.indexFilter.id + 1));
                    }
                }
                if (eb.indexFilter.type === 'Step and Offset') {
                    if (eb.indexFilter.id > 0) {
                        lid = lid.slice(eb.indexFilter.id);
                    }
                    const temp = [];
                    for (let i = 0; i < lid.length; i += eb.indexFilter.step) {
                        temp.push(lid[i]);
                    }
                    lid = temp;
                }
                if (!eb.beatDistribution &&
                    !eb.brightnessDistribution &&
                    !eb.hueDistribution &&
                    !eb.affectFirst) {
                    let previousEvent;
                    let previousBase;
                    eb.events.forEach((ev) => {
                        if (ev.transition === 2 && previousEvent) {
                            if (previousBase.frequency) {
                                for (let t = previousBase.time + 1 / previousBase.frequency; t < ev.time; t += 1 / previousBase.frequency) {
                                    events.push({
                                        ...structuredClone(previousEvent),
                                        time: t - 1 / (previousBase.frequency * 2),
                                    }, {
                                        ...structuredClone(previousEvent),
                                        time: t,
                                    });
                                }
                            }
                            events.push(previousEvent);
                            previousBase = ev;
                            return;
                        }
                        const event = {
                            time: q.time + ev.time,
                            type: q.type,
                            value: this.internalEventValue(ev.color, ev.transition),
                            floatValue: ev.brightness,
                            customData: { ...ev.customData, lightID: lid },
                        };
                        events.push(event);
                        previousEvent = event;
                        previousBase = ev;
                    });
                    return;
                }
                let previousEvent;
                let previousBase;
                let isFirst = !eb.affectFirst;
                const lastEventTime = eb.events.at(-1)?.time ?? 0;
                eb.events.forEach((ev) => {
                    for (let i = 0, x = 0; i < lid.length; i++,
                        x = eb.beatDistributionType === 'Division'
                            ? easings_js_1.EasingsFn[eb.beatDistributionEasing ?? 'easeLinear'](i / (lid.length - 1)) * eb.beatDistribution
                            : i * (lastEventTime + eb.beatDistribution)) {
                        if (ev.transition === 2 && previousEvent) {
                            if (previousBase.frequency) {
                                for (let t = previousBase.time + 1 / previousBase.frequency; t < ev.time; t += 1 / previousBase.frequency) {
                                    events.push({
                                        ...structuredClone(previousEvent),
                                        time: t - 1 / (previousBase.frequency * 2) + x,
                                    }, {
                                        ...structuredClone(previousEvent),
                                        time: t,
                                    });
                                }
                            }
                            else {
                                events.push(previousEvent);
                            }
                            previousBase = ev;
                            return;
                        }
                        if (previousBase?.frequency) {
                            for (let t = previousBase.time + 1 / previousBase.frequency; t < ev.time; t += 1 / previousBase.frequency) {
                                events.push({
                                    ...structuredClone(previousEvent),
                                    time: t - 1 / (previousBase.frequency * 2),
                                }, {
                                    ...structuredClone(previousEvent),
                                    time: t,
                                });
                            }
                        }
                        const event = {
                            time: q.time + ev.time + x,
                            type: q.type,
                            value: this.internalEventValue(ev.color, ev.transition),
                            floatValue: Math.max(isFirst
                                ? ev.brightness
                                : eb.brightnessDistributionType === 'Division'
                                    ? ev.brightness +
                                        easings_js_1.EasingsFn[eb.brightnessDistributionEasing ?? 'easeLinear'](i / (lid.length - 1)) *
                                            eb.brightnessDistribution
                                    : ev.brightness + i * eb.brightnessDistribution, 0),
                            customData: { ...ev.customData, lightID: lid[i] },
                        };
                        if (eb.hueDistribution && (0, basicEvent_js_1.isLightEventType)(event.type)) {
                            if (!event.customData.color) {
                                event.customData.color = (0, basicEvent_js_1.isWhiteEventValue)(event.value)
                                    ? [1, 1, 1]
                                    : (0, convertor_js_1.colorFrom)(colorScheme_js_1.ColorScheme[colorScheme_js_1.EnvironmentSchemeName[this.environment]][(0, basicEvent_js_1.isRedEventValue)(event.value) ? '_envColorLeft' : '_envColorRight']);
                            }
                            if (!isFirst) {
                                event.customData.color = (0, convertor_js_1.hsvaToRgba)((0, convertor_js_1.rgbaToHsva)(event.customData.color).map((v, x) => {
                                    if (!x) {
                                        v += eb.hueDistributionType === 'Division'
                                            ? easings_js_1.EasingsFn[eb.hueDistributionEasing ??
                                                'easeLinear'](i / (lid.length - 1)) *
                                                eb.hueDistribution
                                            : i * eb.hueDistribution;
                                    }
                                    return v;
                                }));
                            }
                        }
                        events.push(event);
                        previousEvent = event;
                        previousBase = ev;
                    }
                    isFirst = false;
                });
            });
        }
        if (overwrite) {
            mapData.lightshow.basicEvents = [];
        }
        mapData.lightshow.basicEvents.push(...events);
    }
}
exports.LightMapper = LightMapper;
