"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isEndNote = isEndNote;
exports.isIntersectNote = isIntersectNote;
exports.predictDirection = predictDirection;
exports.checkDirection = checkDirection;
const baseNote_js_1 = require("../../beatmap/helpers/core/baseNote.js");
const gridObject_js_1 = require("../../beatmap/helpers/core/gridObject.js");
const constants_js_1 = require("../../beatmap/shared/constants.js");
const trigonometry_js_1 = require("../../utils/math/trigonometry.js");
function angleInRange(alpha, lower, upper) {
    return (0, trigonometry_js_1.mod)(alpha - lower, 360) <= (0, trigonometry_js_1.mod)(upper - lower, 360);
}
function getAngle(from, to) {
    return (0, trigonometry_js_1.radToDeg)(Math.atan2(to[1] - from[1], to[0] - from[0]));
}
/** Check if current note can be swung into from previous note. */
function isEndNote(endNote, startNote, maybeDirection, angleTolerance = 85) {
    const endPos = (0, gridObject_js_1.resolveGridPosition)(endNote);
    const startPos = (0, gridObject_js_1.resolveGridPosition)(startNote);
    const angleFromStart = getAngle(startPos, endPos);
    const angleFromEnd = getAngle(endPos, startPos);
    if (startNote.direction === constants_js_1.NoteDirection.ANY) {
        const dirAngle = (0, trigonometry_js_1.mod)((0, baseNote_js_1.resolveNoteAngle)(endNote.direction) + 180, 360);
        return angleInRange(angleFromEnd, dirAngle - angleTolerance, dirAngle + angleTolerance);
    }
    if (maybeDirection !== constants_js_1.NoteDirection.ANY) {
        const dirAngle = (0, baseNote_js_1.resolveNoteAngle)(maybeDirection);
        return (angleInRange(angleFromStart, dirAngle - angleTolerance, dirAngle + angleTolerance) ||
            angleInRange(angleFromEnd, dirAngle - angleTolerance, dirAngle + angleTolerance));
    }
    const dirAngle = (0, baseNote_js_1.resolveNoteAngle)(startNote.direction);
    return angleInRange(angleFromStart, dirAngle - angleTolerance, dirAngle + angleTolerance);
}
function hasIntersect(currNote, compareTo, angleDistances, ahead = false) {
    if (currNote.direction !== constants_js_1.NoteDirection.ANY) {
        return false;
    }
    const [nX1, nY1] = (0, gridObject_js_1.resolveGridPosition)(currNote);
    const [nX2, nY2] = (0, gridObject_js_1.resolveGridPosition)(compareTo);
    const hackValue = ahead ? 540 : 360;
    const directionAngle = (0, baseNote_js_1.resolveNoteAngle)(currNote.direction);
    const noteLookAngle = ((0, trigonometry_js_1.radToDeg)(Math.atan2(nY1 - nY2, nX1 - nX2)) + 450) % 360;
    return angleDistances.some(([angleRange, maxDistance, offset]) => {
        offset = offset ?? 0;
        const angleStart = (directionAngle + hackValue - angleRange + offset) % 360;
        const angleEnd = (directionAngle + hackValue + angleRange + offset) % 360;
        return (maxDistance >=
            Math.sqrt(Math.pow(nX1 - nX2, 2) + Math.pow(nY1 - nY2, 2)) &&
            ((angleStart < angleEnd &&
                angleStart <= noteLookAngle &&
                noteLookAngle <= angleEnd) ||
                (angleStart >= angleEnd &&
                    (noteLookAngle <= angleEnd || noteLookAngle >= angleStart))));
    });
}
/**
 * Check if the note intersect on swing path by angle and distance.
 *
 * Can be used to check for hitbox collision or handclap.
 *
 * ```ts
 * if (isIntersect(note1, note2, [[20, 1.5]])) {}
 * ```
 */
function isIntersectNote(noteA, noteB, angleDistances, ahead = false) {
    return [
        hasIntersect(noteA, noteB, angleDistances, ahead),
        hasIntersect(noteB, noteA, angleDistances, ahead),
    ];
}
// TODO: update with new position/rotation system
function predictDirection(endNote, startNote) {
    if (isEndNote(endNote, startNote, constants_js_1.NoteDirection.ANY)) {
        return endNote.direction === constants_js_1.NoteDirection.ANY ? startNote.direction : endNote.direction;
    }
    if (endNote.direction !== constants_js_1.NoteDirection.ANY) {
        return endNote.direction;
    }
    if (endNote.time > startNote.time) {
        // if end note on right side
        if (endNote.posX > startNote.posX) {
            if ((0, gridObject_js_1.isHorizontal)(endNote, startNote)) {
                return constants_js_1.NoteDirection.RIGHT;
            }
        }
        // if end note on left side
        if (endNote.posX < startNote.posX) {
            if ((0, gridObject_js_1.isHorizontal)(endNote, startNote)) {
                return constants_js_1.NoteDirection.LEFT;
            }
        }
        // if end note is above
        if (endNote.posY > startNote.posY) {
            if ((0, gridObject_js_1.isVertical)(endNote, startNote)) {
                return constants_js_1.NoteDirection.UP;
            }
            if (endNote.posX > startNote.posX) {
                return constants_js_1.NoteDirection.UP_RIGHT;
            }
            if (endNote.posX < startNote.posX) {
                return constants_js_1.NoteDirection.UP_LEFT;
            }
        }
        // if end note is below
        if (endNote.posY < startNote.posY) {
            if ((0, gridObject_js_1.isVertical)(endNote, startNote)) {
                return constants_js_1.NoteDirection.DOWN;
            }
            if (endNote.posX > startNote.posX) {
                return constants_js_1.NoteDirection.DOWN_RIGHT;
            }
            if (endNote.posX < startNote.posX) {
                return constants_js_1.NoteDirection.DOWN_LEFT;
            }
        }
    }
    return constants_js_1.NoteDirection.ANY;
}
/**
 * Check the angle equality of the two notes.
 * @param {(T|number|null)} noteA - First beatmap note, note `direction`, or null value
 * @param {(T|number|null)} noteB - Second beatmap note, note `direction`, or null value
 * @param {number} angleTol - Angle tolerance
 * @param {boolean} equal - If it should check inner or outer angle
 * @returns {boolean} If condition is met
 */
function checkDirection(noteA, noteB, angleTol, equal) {
    let noteAngleA;
    let noteAngleB;
    if (noteA === null || noteB === null) {
        return false;
    }
    if (typeof noteA === 'number') {
        noteAngleA = noteA;
    }
    else {
        if (noteA.direction === constants_js_1.NoteDirection.ANY) {
            return false;
        }
        noteAngleA = (0, baseNote_js_1.resolveNoteAngle)(noteA.direction);
    }
    if (typeof noteB === 'number') {
        noteAngleB = noteB;
    }
    else {
        if (noteB.direction === constants_js_1.NoteDirection.ANY) {
            return false;
        }
        noteAngleB = (0, baseNote_js_1.resolveNoteAngle)(noteB.direction);
    }
    return equal
        ? (0, trigonometry_js_1.shortRotDistance)(noteAngleA, noteAngleB, 360) <= angleTol
        : (0, trigonometry_js_1.shortRotDistance)(noteAngleA, noteAngleB, 360) >= angleTol;
}
