"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixBoolean = fixBoolean;
exports.fixInt = fixInt;
exports.fixFloat = fixFloat;
exports.fixString = fixString;
exports.fixStringAry = fixStringAry;
exports.fixVector2 = fixVector2;
exports.fixVector3 = fixVector3;
exports.fixVector4 = fixVector4;
exports.fixColor = fixColor;
exports.fixPointDefinition1Base = fixPointDefinition1Base;
exports.fixPointDefinition3Base = fixPointDefinition3Base;
exports.fixPointDefinition4Base = fixPointDefinition4Base;
const easings_js_1 = require("../../utils/math/easings.js");
const helpers_js_1 = require("../../utils/math/helpers.js");
function fixBoolean(value, defaultValue) {
    switch (typeof value) {
        case 'boolean':
            return value;
        case 'number':
            if (value)
                return true;
            return false;
        case 'string': {
            if (value.toLowerCase() === 'true')
                return true;
            if (value.toLowerCase() === 'false')
                return false;
            if (typeof defaultValue === 'boolean')
                return defaultValue;
            throw new TypeError(`Could not evaluate boolean for ${value}`);
        }
        default:
            throw new TypeError(`Could not determine type of value ${value}`);
    }
}
function fixInt(value, defaultValue, range) {
    if (typeof defaultValue === 'number' && !Number.isInteger(defaultValue)) {
        throw new TypeError(`Default value must be integer; received ${value}`);
    }
    if (Array.isArray(defaultValue) &&
        defaultValue.some((dv) => !Number.isInteger(dv))) {
        throw new TypeError(`Default value in array must be integer; received ${defaultValue}`);
    }
    if (range && range.some((n) => !Number.isInteger(n))) {
        throw new TypeError(`Range value must be integer; received ${range}`);
    }
    switch (typeof value) {
        case 'number':
            if (isNaN(value)) {
                if (range && !range.includes(value)) {
                    if (typeof defaultValue === 'number') {
                        value = defaultValue;
                    }
                    else if (Array.isArray(defaultValue)) {
                        value = (0, helpers_js_1.clamp)(value, defaultValue[0], defaultValue[1]);
                    }
                    else {
                        value = (0, helpers_js_1.clamp)(value, range.at(0) ?? 0, range.at(-1) ?? 0);
                    }
                    return value;
                }
                throw new TypeError(`Could not evaluate number for ${value}`);
            }
            value = Math.round(value);
            if (range && !range.includes(value)) {
                if (typeof defaultValue === 'number') {
                    value = defaultValue;
                }
                else if (Array.isArray(defaultValue)) {
                    value = (0, helpers_js_1.clamp)(value, defaultValue[0], defaultValue[1]);
                }
                else {
                    value = (0, helpers_js_1.clamp)(value, range.at(0) ?? 0, range.at(-1) ?? 0);
                }
            }
            return value;
        case 'string': {
            const val = parseInt(value);
            if (isNaN(val)) {
                throw new TypeError(`Could not evaluate number for ${value}`);
            }
            if (range && !range.includes(val)) {
                if (typeof defaultValue === 'number') {
                    value = defaultValue;
                }
                else if (Array.isArray(defaultValue)) {
                    value = (0, helpers_js_1.clamp)(val, defaultValue[0], defaultValue[1]);
                }
                else {
                    value = (0, helpers_js_1.clamp)(val, range.at(0) ?? 0, range.at(-1) ?? 0);
                }
            }
            return val;
        }
        case 'boolean':
            if (typeof defaultValue === 'number') {
                value = defaultValue;
            }
            else if (Array.isArray(defaultValue)) {
                value = defaultValue[0];
            }
            else if (range) {
                value = range.at(0) ?? 0;
            }
            throw new TypeError('Could not evaluate float value from boolean');
        default:
            if (typeof defaultValue === 'number') {
                value = defaultValue;
            }
            else if (Array.isArray(defaultValue)) {
                value = defaultValue[0];
            }
            else if (range) {
                value = range.at(0) ?? 0;
            }
            throw new TypeError(`Could not determine type of value ${value}`);
    }
}
function fixFloat(value, defaultValue, min, max) {
    if (typeof defaultValue === 'number' && isNaN(defaultValue)) {
        throw new TypeError(`Default value must not be NaN`);
    }
    switch (typeof value) {
        case 'number':
            if (typeof min === 'number' && typeof max === 'number') {
                return (0, helpers_js_1.clamp)(value, min, max);
            }
            return value;
        case 'string': {
            const val = parseFloat(value);
            if (isNaN(val)) {
                throw new TypeError(`Could not evaluate number for ${value}`);
            }
            if (typeof min === 'number' && typeof max === 'number') {
                return (0, helpers_js_1.clamp)(val, min, max);
            }
            return val;
        }
        case 'boolean':
            if (typeof defaultValue === 'number') {
                return defaultValue;
            }
            throw new TypeError('Could not evaluate float value from boolean');
        default:
            if (typeof defaultValue === 'number') {
                return defaultValue;
            }
            throw new TypeError(`Could not determine type of value ${value}`);
    }
}
function fixString(value, defaultValue) {
    if (typeof defaultValue === 'string' && !defaultValue.trim()) {
        throw new TypeError(`Default string cannot be empty`);
    }
    switch (typeof value) {
        case 'string':
            return value;
        case 'number':
            return value.toString();
        case 'boolean':
            if (typeof defaultValue === 'string') {
                return defaultValue;
            }
            throw new TypeError('Could not evaluate string value from boolean');
        default:
            if (typeof defaultValue === 'string') {
                return defaultValue;
            }
            throw new TypeError(`Could not determine type of value ${value}`);
    }
}
/**
 * Fix string array value by interpreting unknown value as string array or return default value.
 */
function fixStringAry(value, defaultValue) {
    return value.map((v) => fixString(v, defaultValue));
}
const easingsList = Object.keys(easings_js_1.EasingsFn);
/**
 * Fix vector value by interpreting unknown value as vector or return default value.
 */
function fixVector2(value, defaultValue) {
    return Array.isArray(value)
        ? [
            fixFloat(value.at(0), defaultValue[0]),
            fixFloat(value.at(1), defaultValue[1]),
        ]
        : defaultValue;
}
/**
 * Fix vector value by interpreting unknown value as vector or return default value.
 */
function fixVector3(value, defaultValue) {
    return Array.isArray(value)
        ? [
            fixFloat(value.at(0), defaultValue[0]),
            fixFloat(value.at(1), defaultValue[1]),
            fixFloat(value.at(2), defaultValue[2]),
        ]
        : defaultValue;
}
/**
 * Fix vector value by interpreting unknown value as vector or return default value.
 */
function fixVector4(value, defaultValue) {
    return Array.isArray(value)
        ? [
            fixFloat(value.at(0), defaultValue[0]),
            fixFloat(value.at(1), defaultValue[1]),
            fixFloat(value.at(2), defaultValue[2]),
            fixFloat(value.at(3), defaultValue[3]),
        ]
        : defaultValue;
}
/**
 * Fix color value by interpreting unknown value as color or return default value.
 */
function fixColor(value, defaultValue) {
    return Array.isArray(value)
        ? [
            fixFloat(value.at(0), defaultValue[0]),
            fixFloat(value.at(1), defaultValue[1]),
            fixFloat(value.at(2), defaultValue[2]),
            fixFloat(value.at(3), defaultValue.at(3) ?? 1),
        ]
        : defaultValue;
}
/**
 * Fix float point definition value by interpreting unknown value as float point definition or return default value.
 */
function fixPointDefinition1Base(value, defaultValue) {
    return Array.isArray(value)
        ? typeof value.at(0) === 'string'
            ? value.map((elm, i) => {
                if (i === 0)
                    return elm;
                if (Array.isArray(elm)) {
                    return elm.map((x) => {
                        if (typeof x === 'string')
                            return x;
                        return fixFloat(x, defaultValue);
                    });
                }
                return elm;
            })
            : value
                .filter((ary) => Array.isArray(ary))
                .map((elm) => {
                const temp = [
                    fixFloat(elm.at(0), defaultValue),
                    fixFloat(elm.at(1), 1, 0, 1),
                ];
                if (elm.length > 2) {
                    const attr = elm
                        .slice(3)
                        .filter((e) => typeof e === 'string');
                    const ease = attr.find((e) => easingsList.includes(e));
                    const spline = attr.find((e) => e === 'splineCatmullRom');
                    let idx = 2;
                    if (ease) {
                        temp[idx++] = ease;
                    }
                    if (spline) {
                        temp[idx++] = spline;
                    }
                }
                return temp;
            })
        : [];
}
/**
 * Fix vector point definition value by interpreting unknown value as vector point definition or return default value.
 */
function fixPointDefinition3Base(value, defaultValue) {
    return Array.isArray(value)
        ? typeof value.at(0) === 'string'
            ? value.map((elm, i) => {
                if (i === 0)
                    return elm;
                if (Array.isArray(elm)) {
                    return elm.map((x, j) => {
                        if (typeof x === 'string')
                            return x;
                        return fixFloat(x, defaultValue[j]);
                    });
                }
                return elm;
            })
            : value
                .filter((ary) => Array.isArray(ary))
                .map((elm) => {
                const temp = [
                    fixFloat(elm.at(0), defaultValue[0]),
                    fixFloat(elm.at(1), defaultValue[1]),
                    fixFloat(elm.at(2), defaultValue[2]),
                    fixFloat(elm.at(3), 1, 0, 1),
                ];
                if (elm.length > 4) {
                    const attr = elm
                        .slice(4)
                        .filter((e) => typeof e === 'string');
                    const ease = attr.find((e) => easingsList.includes(e));
                    const spline = attr.find((e) => e === 'splineCatmullRom');
                    let idx = 4;
                    if (ease) {
                        temp[idx++] = ease;
                    }
                    if (spline) {
                        temp[idx++] = spline;
                    }
                }
                return temp;
            })
        : [];
}
/**
 * Fix vector point definition value by interpreting unknown value as vector point definition or return default value.
 */
function fixPointDefinition4Base(value, defaultValue) {
    return Array.isArray(value)
        ? typeof value.at(0) === 'string'
            ? value.map((elm, i) => {
                if (i === 0)
                    return elm;
                if (Array.isArray(elm)) {
                    return elm.map((x, j) => {
                        if (typeof x === 'string')
                            return x;
                        return fixFloat(x, defaultValue[j]);
                    });
                }
                return elm;
            })
            : value
                .filter((ary) => Array.isArray(ary))
                .map((elm) => {
                const temp = [
                    fixFloat(value.at(0), defaultValue[0]),
                    fixFloat(value.at(1), defaultValue[1]),
                    fixFloat(value.at(2), defaultValue[2]),
                    fixFloat(value.at(3), defaultValue[3]),
                    fixFloat(elm.at(4), 1, 0, 1),
                ];
                if (elm.length > 5) {
                    const attr = elm
                        .slice(5)
                        .filter((e) => typeof e === 'string');
                    const ease = attr.find((e) => easingsList.includes(e));
                    const lerp = attr.find((e) => e === 'lerpHSV');
                    let idx = 5;
                    if (ease) {
                        temp[idx++] = ease;
                    }
                    if (lerp) {
                        temp[idx++] = lerp;
                    }
                }
                return temp;
            })
        : [];
}
